@@ -1,7 +1,7 @@
-/* $OpenLDAP: pkg/ldap/libraries/libldap/sortctrl.c,v 1.19 2006/03/31 19:18:04 hyc Exp $ */
+/* $OpenLDAP: pkg/ldap/libraries/libldap/sortctrl.c,v 1.19.2.2 2007/02/13 20:22:12 kurt Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 1998-2006 The OpenLDAP Foundation.
+ * Copyright 1998-2007 The OpenLDAP Foundation.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -27,9 +27,6 @@
  * can be found in the file "build/LICENSE-2.0.1" in this distribution
  * of OpenLDAP Software.
  */
-/* Portions Copyright (C) The Internet Society (1997)
- * ASN.1 fragments are from RFC 2251; see RFC for full legal notices.
- */
 
 #include "portable.h"
 
@@ -296,14 +293,15 @@ ldap_create_sort_control_value(
 	BerElement	*ber = NULL;
 	ber_tag_t	tag;
 
-	if ( ld == NULL || keyList == NULL || value == NULL ) {
-		if ( ld )
-			ld->ld_errno = LDAP_PARAM_ERROR;
+	assert( ld != NULL );
+	assert( LDAP_VALID( ld ) );
+
+	if ( ld == NULL ) return LDAP_PARAM_ERROR;
+	if ( keyList == NULL || value == NULL ) {
+		ld->ld_errno = LDAP_PARAM_ERROR;
 		return LDAP_PARAM_ERROR;
 	}
 
-	assert( LDAP_VALID( ld ) );
-
 	value->bv_val = NULL;
 	value->bv_len = 0;
 
@@ -412,6 +410,10 @@ ldap_create_sort_control(
 	struct berval	value;
 	BerElement	*ber;
 
+	assert( ld != NULL );
+	assert( LDAP_VALID( ld ) );
+
+	if ( ld == NULL ) return LDAP_PARAM_ERROR;
 	if ( ctrlp == NULL ) {
 		ld->ld_errno = LDAP_PARAM_ERROR;
 		return ld->ld_errno;
@@ -496,6 +498,9 @@ ldap_parse_sortresponse_control(
 	ber_tag_t tag, berTag;
 	ber_len_t berLen;
 
+	assert( ld != NULL );
+	assert( LDAP_VALID( ld ) );
+
 	if (ld == NULL) {
 		return LDAP_PARAM_ERROR;
 	}
