@@ -1,8 +1,8 @@
 /* tls.c - Handle tls/ssl using SSLeay or OpenSSL. */
-/* $OpenLDAP: pkg/ldap/libraries/libldap/tls.c,v 1.133.2.4 2006/08/17 23:50:26 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/libraries/libldap/tls.c,v 1.133.2.6 2007/02/13 20:22:12 kurt Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 1998-2006 The OpenLDAP Foundation.
+ * Copyright 1998-2007 The OpenLDAP Foundation.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -103,7 +103,10 @@ static void tls_init_threads( void )
 		ldap_pvt_thread_mutex_init( &tls_mutexes[i] );
 	}
 	CRYPTO_set_locking_callback( tls_locking_cb );
-	/* FIXME: the thread id should be added somehow... */
+	CRYPTO_set_id_callback( ldap_pvt_thread_self );
+	/* FIXME: CRYPTO_set_id_callback only works when ldap_pvt_thread_t
+	 * is an integral type that fits in an unsigned long
+	 */
 
 	ldap_pvt_thread_mutex_init( &tls_def_ctx_mutex );
 	ldap_pvt_thread_mutex_init( &tls_connect_mutex );
@@ -215,6 +218,8 @@ ldap_int_tls_init_ctx( struct ldapoptions *lo, int is_server )
 	if ( lo->ldo_tls_ctx )
 		return 0;
 
+	ldap_pvt_tls_init();
+
 	if ( is_server && !certfile && !keyfile && !cacertfile && !cacertdir ) {
 		/* minimum configuration not provided */
 		return LDAP_NOT_SUPPORTED;
@@ -594,7 +599,7 @@ sb_tls_read( Sockbuf_IO_Desc *sbiod, void *buf, ber_len_t len)
 	err = SSL_get_error( p->ssl, ret );
 	if (err == SSL_ERROR_WANT_READ ) {
 		sbiod->sbiod_sb->sb_trans_needs_read = 1;
-		errno = EWOULDBLOCK;
+		sock_errset(EWOULDBLOCK);
 	}
 	else
 		sbiod->sbiod_sb->sb_trans_needs_read = 0;
@@ -620,7 +625,7 @@ sb_tls_write( Sockbuf_IO_Desc *sbiod, void *buf, ber_len_t len)
 	err = SSL_get_error( p->ssl, ret );
 	if (err == SSL_ERROR_WANT_WRITE ) {
 		sbiod->sbiod_sb->sb_trans_needs_write = 1;
-		errno = EWOULDBLOCK;
+		sock_errset(EWOULDBLOCK);
 
 	} else {
 		sbiod->sbiod_sb->sb_trans_needs_write = 0;
@@ -676,7 +681,7 @@ sb_tls_bio_read( BIO *b, char *buf, int len )
 
 	BIO_clear_retry_flags( b );
 	if ( ret < 0 ) {
-		int err = errno;
+		int err = sock_errno();
 		if ( err == EAGAIN || err == EWOULDBLOCK ) {
 			BIO_set_retry_read( b );
 		}
@@ -703,7 +708,7 @@ sb_tls_bio_write( BIO *b, const char *buf, int len )
 
 	BIO_clear_retry_flags( b );
 	if ( ret < 0 ) {
-		int err = errno;
+		int err = sock_errno();
 		if ( err == EAGAIN || err == EWOULDBLOCK ) {
 			BIO_set_retry_write( b );
 		}
@@ -1294,6 +1299,10 @@ ldap_pvt_tls_get_option( LDAP *ld, int option, void *arg )
 		*(int *)arg = lo->ldo_tls_crlcheck;
 		break;
 #endif
+	case LDAP_OPT_X_TLS_CIPHER_SUITE:
+		*(char **)arg = lo->ldo_tls_ciphersuite ?
+			LDAP_STRDUP( lo->ldo_tls_ciphersuite ) : NULL;
+		break;
 	case LDAP_OPT_X_TLS_RANDOM_FILE:
 		*(char **)arg = tls_opt_randfile ?
 			LDAP_STRDUP( tls_opt_randfile ) : NULL;
