@@ -1,7 +1,7 @@
-/* $OpenLDAP: pkg/ldap/libraries/libldap_r/tpool.c,v 1.52.2.1 2006/05/15 17:04:40 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/libraries/libldap_r/tpool.c,v 1.52.2.3 2007/02/13 20:22:13 kurt Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 1998-2006 The OpenLDAP Foundation.
+ * Copyright 1998-2007 The OpenLDAP Foundation.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -464,23 +464,110 @@ ldap_pvt_thread_pool_maxthreads ( ldap_pvt_thread_pool_t *tpool, int max_threads
 }
 
 int
-ldap_pvt_thread_pool_backload ( ldap_pvt_thread_pool_t *tpool )
+ldap_pvt_thread_pool_query ( ldap_pvt_thread_pool_t *tpool, ldap_pvt_thread_pool_param_t param, void *value )
 {
-	struct ldap_int_thread_pool_s *pool;
-	int count;
+	struct ldap_int_thread_pool_s	*pool;
+	int				count = -1;
 
-	if (tpool == NULL)
-		return(-1);
+	if ( tpool == NULL || value == NULL ) {
+		return -1;
+	}
 
 	pool = *tpool;
 
-	if (pool == NULL)
-		return(0);
+	if ( pool == NULL ) {
+		return 0;
+	}
 
 	ldap_pvt_thread_mutex_lock(&pool->ltp_mutex);
-	count = pool->ltp_pending_count + pool->ltp_active_count;
-	ldap_pvt_thread_mutex_unlock(&pool->ltp_mutex);
-	return(count);
+	switch ( param ) {
+	case LDAP_PVT_THREAD_POOL_PARAM_MAX:
+		count = pool->ltp_max_count;
+		break;
+
+	case LDAP_PVT_THREAD_POOL_PARAM_MAX_PENDING:
+		count = pool->ltp_max_pending;
+		break;
+
+	case LDAP_PVT_THREAD_POOL_PARAM_OPEN:
+		count = pool->ltp_open_count;
+		break;
+
+	case LDAP_PVT_THREAD_POOL_PARAM_STARTING:
+		count = pool->ltp_starting;
+		break;
+
+	case LDAP_PVT_THREAD_POOL_PARAM_ACTIVE:
+		count = pool->ltp_active_count;
+		break;
+
+	case LDAP_PVT_THREAD_POOL_PARAM_PENDING:
+		count = pool->ltp_pending_count;
+		break;
+
+	case LDAP_PVT_THREAD_POOL_PARAM_BACKLOAD:
+		count = pool->ltp_pending_count + pool->ltp_active_count;
+		break;
+
+	case LDAP_PVT_THREAD_POOL_PARAM_ACTIVE_MAX:
+		break;
+
+	case LDAP_PVT_THREAD_POOL_PARAM_PENDING_MAX:
+		break;
+
+	case LDAP_PVT_THREAD_POOL_PARAM_BACKLOAD_MAX:
+		break;
+
+	case LDAP_PVT_THREAD_POOL_PARAM_STATE: {
+		static struct {
+			char				*name;
+			ldap_int_thread_pool_state_t	state;
+		}		str2state[] = {
+			{ "running",	LDAP_INT_THREAD_POOL_RUNNING },
+			{ "finishing",	LDAP_INT_THREAD_POOL_FINISHING },
+			{ "stopping",	LDAP_INT_THREAD_POOL_STOPPING },
+			{ "pausing",	LDAP_INT_THREAD_POOL_PAUSING },
+			{ NULL }
+		};
+		int		i;
+
+		for ( i = 0; str2state[ i ].name != NULL; i++ ) {
+			if ( str2state[ i ].state == pool->ltp_state ) {
+				break;
+			}
+		}
+		*((char **)value) = str2state[ i ].name;
+		if ( str2state[ i ].name != NULL ) {
+			count = -2;
+		}
+		} break;
+	}
+	ldap_pvt_thread_mutex_unlock( &pool->ltp_mutex );
+
+	if ( count > -1 ) {
+		*((int *)value) = count;
+	}
+
+	return ( count == -1 ? -1 : 0 );
+}
+
+/*
+ * wrapper for ldap_pvt_thread_pool_query(), left around
+ * for backwards compatibility
+ */
+int
+ldap_pvt_thread_pool_backload ( ldap_pvt_thread_pool_t *tpool )
+{
+	int	rc, count;
+
+	rc = ldap_pvt_thread_pool_query( tpool,
+		LDAP_PVT_THREAD_POOL_PARAM_BACKLOAD, (void *)&count );
+
+	if ( rc == 0 ) {
+		return count;
+	}
+
+	return rc;
 }
 
 int
@@ -633,12 +720,7 @@ ldap_int_thread_pool_wrapper (
 		}
 	}
 
-	for ( i=0; i<MAXKEYS && uctx.ltu_key[i].ltk_key; i++ ) {
-		if (uctx.ltu_key[i].ltk_free)
-			uctx.ltu_key[i].ltk_free(
-				uctx.ltu_key[i].ltk_key,
-				uctx.ltu_key[i].ltk_data );
-	}
+	ldap_pvt_thread_pool_context_reset(&uctx);
 
 	thread_keys[keyslot].ctx = NULL;
 	thread_keys[keyslot].id = tid_zero;
@@ -781,7 +863,7 @@ void ldap_pvt_thread_pool_purgekey( void *key )
 /*
  * This is necessary if the caller does not have access to the
  * thread context handle (for example, a slapd plugin calling
- * slapi_search_internal()). No doubt it is more efficient to
+ * slapi_search_internal()). No doubt it is more efficient
  * for the application to keep track of the thread context
  * handles itself.
  */
@@ -808,7 +890,9 @@ void ldap_pvt_thread_pool_context_reset( void *vctx )
 	ldap_int_thread_userctx_t *ctx = vctx;
 	int i;
 
-	for ( i=0; i<MAXKEYS && ctx->ltu_key[i].ltk_key; i++) {
+	for ( i=MAXKEYS-1; i>=0; i--) {
+		if ( !ctx->ltu_key[i].ltk_key )
+			continue;
 		if ( ctx->ltu_key[i].ltk_free )
 			ctx->ltu_key[i].ltk_free( ctx->ltu_key[i].ltk_key,
 			ctx->ltu_key[i].ltk_data );
