@@ -1,7 +1,7 @@
-/* $OpenLDAP: pkg/ldap/servers/slapd/back-meta/conn.c,v 1.86.2.4 2006/08/17 23:50:31 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/back-meta/conn.c,v 1.86.2.6 2007/02/13 20:22:25 kurt Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 1999-2006 The OpenLDAP Foundation.
+ * Copyright 1999-2007 The OpenLDAP Foundation.
  * Portions Copyright 2001-2003 Pierangelo Masarati.
  * Portions Copyright 1999-2003 Howard Chu.
  * All rights reserved.
@@ -34,13 +34,6 @@
 #include "../back-ldap/back-ldap.h"
 #include "back-meta.h"
 
-/*
- * Set PRINT_CONNTREE larger than 0 to dump the connection tree (debug only)
- */
-#ifndef PRINT_CONNTREE
-#define PRINT_CONNTREE 0
-#endif /* !PRINT_CONNTREE */
-
 /*
  * meta_back_conndn_cmp
  *
@@ -145,9 +138,9 @@ meta_back_conndn_dup(
 /*
  * Debug stuff (got it from libavl)
  */
-#if PRINT_CONNTREE > 0
+#if META_BACK_PRINT_CONNTREE > 0
 static void
-ravl_print( Avlnode *root, int depth )
+meta_back_ravl_print( Avlnode *root, int depth )
 {
 	int     	i;
 	metaconn_t	*mc;
@@ -156,37 +149,68 @@ ravl_print( Avlnode *root, int depth )
 		return;
 	}
 	
-	ravl_print( root->avl_right, depth + 1 );
+	meta_back_ravl_print( root->avl_right, depth + 1 );
 	
 	for ( i = 0; i < depth; i++ ) {
 		fprintf( stderr, "-" );
 	}
 
 	mc = (metaconn_t *)root->avl_data;
-	fprintf( stderr, "mc=%p local=\"%s\" conn=%p %s refcnt=%d\n",
+	fprintf( stderr, "mc=%p local=\"%s\" conn=%p %s refcnt=%d%s\n",
 		(void *)mc,
 		mc->mc_local_ndn.bv_val ? mc->mc_local_ndn.bv_val : "",
 		(void *)mc->mc_conn,
-		avl_bf2str( root->avl_bf ), mc->mc_refcnt );
+		avl_bf2str( root->avl_bf ), mc->mc_refcnt,
+		LDAP_BACK_CONN_TAINTED( mc ) ? " tainted" : "" );
 	
-	ravl_print( root->avl_left, depth + 1 );
+	meta_back_ravl_print( root->avl_left, depth + 1 );
 }
 
-static void
-myprint( Avlnode *root, char *msg )
+/* NOTE: duplicate from back-ldap/bind.c */
+static char* priv2str[] = {
+	"privileged",
+	"privileged/TLS",
+	"anonymous",
+	"anonymous/TLS",
+	"bind",
+	"bind/TLS",
+	NULL
+};
+
+void
+meta_back_print_conntree( metainfo_t *mi, char *msg )
 {
+	int	c;
+
 	fprintf( stderr, "========> %s\n", msg );
 	
-	if ( root == 0 ) {
-		fprintf( stderr, "\tNULL\n" );
+	for ( c = LDAP_BACK_PCONN_FIRST; c < LDAP_BACK_PCONN_LAST; c++ ) {
+		int		i = 0;
+		metaconn_t	*mc;
+
+		fprintf( stderr, "  %s[%d]\n", priv2str[ c ], mi->mi_conn_priv[ c ].mic_num );
+
+		LDAP_TAILQ_FOREACH( mc, &mi->mi_conn_priv[ c ].mic_priv, mc_q )
+		{
+			fprintf( stderr, "    [%d] mc=%p local=\"%s\" conn=%p refcnt=%d flags=0x%08x\n",
+				i,
+				(void *)mc,
+				mc->mc_local_ndn.bv_val ? mc->mc_local_ndn.bv_val : "",
+				(void *)mc->mc_conn, mc->mc_refcnt, mc->msc_mscflags );
+			i++;
+		}
+	}
+	
+	if ( mi->mi_conninfo.lai_tree == NULL ) {
+		fprintf( stderr, "\t(empty)\n" );
 
 	} else {
-		ravl_print( root, 0 );
+		meta_back_ravl_print( mi->mi_conninfo.lai_tree, 0 );
 	}
 	
 	fprintf( stderr, "<======== %s\n", msg );
 }
-#endif /* PRINT_CONNTREE */
+#endif /* META_BACK_PRINT_CONNTREE */
 /*
  * End of debug stuff
  */
@@ -202,20 +226,18 @@ metaconn_alloc(
 {
 	metainfo_t	*mi = ( metainfo_t * )op->o_bd->be_private;
 	metaconn_t	*mc;
-	int		i, ntargets = mi->mi_ntargets;
+	int		ntargets = mi->mi_ntargets;
 
 	assert( ntargets > 0 );
 
 	/* malloc all in one */
 	mc = ( metaconn_t * )ch_calloc( 1, sizeof( metaconn_t )
-			+ sizeof( metasingleconn_t ) * ntargets );
+		+ sizeof( metasingleconn_t ) * ( ntargets - 1 ) );
 	if ( mc == NULL ) {
 		return NULL;
 	}
 
-	for ( i = 0; i < ntargets; i++ ) {
-		mc->mc_conns[ i ].msc_info = mi;
-	}
+	mc->mc_info = mi;
 
 	mc->mc_authz_target = META_BOUND_NONE;
 	mc->mc_refcnt = 1;
@@ -235,7 +257,8 @@ meta_back_init_one_conn(
 	metaconn_t		*mc,
 	int			candidate,
 	int			ispriv,
-	ldap_back_send_t	sendok )
+	ldap_back_send_t	sendok,
+	int			dolock )
 {
 	metainfo_t		*mi = ( metainfo_t * )op->o_bd->be_private;
 	metatarget_t		*mt = mi->mi_targets[ candidate ];
@@ -243,6 +266,7 @@ meta_back_init_one_conn(
 	int			version;
 	dncookie		dc;
 	int			isauthz = ( candidate == mc->mc_authz_target );
+	int			do_return = 0;
 #ifdef HAVE_TLS
 	int			is_ldaps = 0;
 #endif /* HAVE_TLS */
@@ -255,42 +279,99 @@ meta_back_init_one_conn(
 		slap_retry_info_t	*ri = &mt->mt_quarantine;
 		int			dont_retry = 1;
 
-		if ( mt->mt_isquarantined == LDAP_BACK_FQ_YES ) {
-			dont_retry = ( ri->ri_num[ ri->ri_idx ] == SLAP_RETRYNUM_TAIL
-				|| slap_get_time() < ri->ri_last + ri->ri_interval[ ri->ri_idx ] );
-			if ( !dont_retry ) {
-				if ( LogTest( LDAP_DEBUG_ANY ) ) {
-					char	buf[ SLAP_TEXT_BUFLEN ];
+		if ( mt->mt_quarantine.ri_interval ) {
+			ldap_pvt_thread_mutex_lock( &mt->mt_quarantine_mutex );
+			if ( mt->mt_isquarantined == LDAP_BACK_FQ_YES ) {
+				dont_retry = ( ri->ri_num[ ri->ri_idx ] == SLAP_RETRYNUM_TAIL
+					|| slap_get_time() < ri->ri_last + ri->ri_interval[ ri->ri_idx ] );
+				if ( !dont_retry ) {
+					if ( LogTest( LDAP_DEBUG_ANY ) ) {
+						char	buf[ SLAP_TEXT_BUFLEN ];
 
-					snprintf( buf, sizeof( buf ),
-						"meta_back_init_one_conn[%d]: quarantine "
-						"retry block #%d try #%d",
-						candidate, ri->ri_idx, ri->ri_count );
-					Debug( LDAP_DEBUG_ANY, "%s %s.\n",
-						op->o_log_prefix, buf, 0 );
+						snprintf( buf, sizeof( buf ),
+							"meta_back_init_one_conn[%d]: quarantine "
+							"retry block #%d try #%d",
+							candidate, ri->ri_idx, ri->ri_count );
+						Debug( LDAP_DEBUG_ANY, "%s %s.\n",
+							op->o_log_prefix, buf, 0 );
+					}
 				}
 
 				mt->mt_isquarantined = LDAP_BACK_FQ_RETRYING;
 			}
+			ldap_pvt_thread_mutex_unlock( &mt->mt_quarantine_mutex );
 		}
 
 		if ( dont_retry ) {
 			rs->sr_err = LDAP_UNAVAILABLE;
 			if ( op->o_conn && ( sendok & LDAP_BACK_SENDERR ) ) {
+				rs->sr_text = "Target is quarantined";
 				send_ldap_result( op, rs );
 			}
 			return rs->sr_err;
 		}
 	}
 
+retry_lock:;
+	if ( dolock ) {
+		ldap_pvt_thread_mutex_lock( &mi->mi_conninfo.lai_mutex );
+	}
+
 	/*
 	 * Already init'ed
 	 */
-	if ( msc->msc_ld != NULL ) {
-		return rs->sr_err = LDAP_SUCCESS;
+	if ( LDAP_BACK_CONN_ISBOUND( msc )
+		|| LDAP_BACK_CONN_ISANON( msc ) )
+	{
+		assert( msc->msc_ld != NULL );
+		rs->sr_err = LDAP_SUCCESS;
+		do_return = 1;
+
+	} else if ( META_BACK_CONN_CREATING( msc )
+		|| LDAP_BACK_CONN_BINDING( msc ) )
+	{
+		if ( !LDAP_BACK_USE_TEMPORARIES( mi ) ) {
+			if ( dolock ) {
+				ldap_pvt_thread_mutex_unlock( &mi->mi_conninfo.lai_mutex );
+			}
+
+			ldap_pvt_thread_yield();
+			goto retry_lock;
+		}
+
+		/* sounds more appropriate */
+		rs->sr_err = LDAP_BUSY;
+		rs->sr_text = "No connections to target are available";
+		do_return = 1;
+
+	} else if ( META_BACK_CONN_INITED( msc ) ) {
+		assert( msc->msc_ld != NULL );
+		rs->sr_err = LDAP_SUCCESS;
+		do_return = 1;
+
+	} else {
+		/*
+		 * creating...
+		 */
+		META_BACK_CONN_CREATING_SET( msc );
 	}
 
-	msc->msc_mscflags = 0;
+	if ( dolock ) {
+		ldap_pvt_thread_mutex_unlock( &mi->mi_conninfo.lai_mutex );
+	}
+
+	if ( do_return ) {
+		if ( rs->sr_err != LDAP_SUCCESS
+			&& op->o_conn
+			&& ( sendok & LDAP_BACK_SENDERR ) )
+		{
+			send_ldap_result( op, rs );
+		}
+
+		return rs->sr_err;
+	}
+
+	assert( msc->msc_ld == NULL );
        
 	/*
 	 * Attempts to initialize the connection to the target ds
@@ -327,8 +408,10 @@ meta_back_init_one_conn(
 
 #ifdef HAVE_TLS
 	/* start TLS ("tls [try-]{start|propagate}" statement) */
-	if ( ( LDAP_BACK_USE_TLS( mi ) || ( op->o_conn->c_is_tls && LDAP_BACK_PROPAGATE_TLS( mi ) ) )
-			&& !is_ldaps )
+	if ( ( LDAP_BACK_USE_TLS( mi )
+		|| ( op->o_conn->c_is_tls
+			&& LDAP_BACK_PROPAGATE_TLS( mi ) ) )
+		&& !is_ldaps )
 	{
 #ifdef SLAP_STARTTLS_ASYNCHRONOUS
 		/*
@@ -347,10 +430,12 @@ meta_back_init_one_conn(
 
 retry:;
 			rc = ldap_result( msc->msc_ld, msgid, LDAP_MSG_ALL, &tv, &res );
-			if ( rc < 0 ) {
+			switch ( rc ) {
+			case -1:
 				rs->sr_err = LDAP_OTHER;
+				break;
 
-			} else if ( rc == 0 ) {
+			case 0:
 				if ( nretries != 0 ) {
 					if ( nretries > 0 ) {
 						nretries--;
@@ -359,18 +444,28 @@ retry:;
 					goto retry;
 				}
 				rs->sr_err = LDAP_OTHER;
+				break;
 
-			} else if ( rc == LDAP_RES_EXTENDED ) {
+			default:
+				/* only touch when activity actually took place... */
+				if ( mi->mi_idle_timeout != 0 && msc->msc_time < op->o_time ) {
+					msc->msc_time = op->o_time;
+				}
+				break;
+			}
+
+			if ( rc == LDAP_RES_EXTENDED ) {
 				struct berval	*data = NULL;
 
-				rs->sr_err = ldap_parse_extended_result( msc->msc_ld, res,
-						NULL, &data, 0 );
+				/* NOTE: right now, data is unused, so don't get it */
+				rs->sr_err = ldap_parse_extended_result( msc->msc_ld,
+					res, NULL, NULL /* &data */ , 0 );
 				if ( rs->sr_err == LDAP_SUCCESS ) {
 					int		err;
 
 					/* FIXME: matched? referrals? response controls? */
-					rs->sr_err = ldap_parse_result( msc->msc_ld, res,
-						&err, NULL, NULL, NULL, NULL, 1 );
+					rs->sr_err = ldap_parse_result( msc->msc_ld,
+						res, &err, NULL, NULL, NULL, NULL, 1 );
 					res = NULL;
 
 					if ( rs->sr_err == LDAP_SUCCESS ) {
@@ -385,15 +480,14 @@ retry:;
 						ldap_install_tls( msc->msc_ld );
 
 					} else if ( rs->sr_err == LDAP_REFERRAL ) {
+						/* FIXME: LDAP_OPERATIONS_ERROR? */
 						rs->sr_err = LDAP_OTHER;
-						rs->sr_text = "unwilling to chase referral returned by Start TLS exop";
+						rs->sr_text = "Unwilling to chase referral "
+							"returned by Start TLS exop";
 					}
 
 					if ( data ) {
-						if ( data->bv_val ) {
-							ber_memfree( data->bv_val );
-						}
-						ber_memfree( data );
+						ber_bvfree( data );
 					}
 				}
 
@@ -417,10 +511,20 @@ retry:;
 		 * of misconfiguration, but also when used in the chain 
 		 * overlay, where the "uri" can be parsed out of a referral */
 		if ( rs->sr_err == LDAP_SERVER_DOWN
-				|| ( rs->sr_err != LDAP_SUCCESS && LDAP_BACK_TLS_CRITICAL( mi ) ) )
+			|| ( rs->sr_err != LDAP_SUCCESS
+				&& LDAP_BACK_TLS_CRITICAL( mi ) ) )
 		{
-			ldap_unbind_ext( msc->msc_ld, NULL, NULL );
-			msc->msc_ld = NULL;
+
+#ifdef DEBUG_205
+			Debug( LDAP_DEBUG_ANY,
+				"### %s meta_back_init_one_conn(TLS) "
+				"ldap_unbind_ext[%d] ld=%p\n",
+				op->o_log_prefix, candidate,
+				(void *)msc->msc_ld );
+#endif /* DEBUG_205 */
+
+			/* need to trash a failed Start TLS */
+			meta_clear_one_candidate( op, mc, candidate );
 			goto error_return;
 		}
 	}
@@ -447,6 +551,10 @@ retry:;
 		if ( !BER_BVISNULL( &mt->mt_idassert_authcDN ) ) {
 			ber_bvreplace( &msc->msc_bound_ndn, &mt->mt_idassert_authcDN );
 			if ( !BER_BVISNULL( &mt->mt_idassert_passwd ) ) {
+				if ( !BER_BVISNULL( &msc->msc_cred ) ) {
+					memset( msc->msc_cred.bv_val, 0,
+						msc->msc_cred.bv_len );
+				}
 				ber_bvreplace( &msc->msc_cred, &mt->mt_idassert_passwd );
 			}
 
@@ -454,8 +562,6 @@ retry:;
 			ber_bvreplace( &msc->msc_bound_ndn, &slap_empty_bv );
 		}
 
-		LDAP_BACK_CONN_ISPRIV_SET( msc );
-
 	} else {
 		if ( !BER_BVISNULL( &msc->msc_cred ) ) {
 			memset( msc->msc_cred.bv_val, 0, msc->msc_cred.bv_len );
@@ -481,8 +587,17 @@ retry:;
 			if ( ldap_back_dn_massage( &dc, &op->o_conn->c_dn,
 						&msc->msc_bound_ndn ) )
 			{
-				ldap_unbind_ext( msc->msc_ld, NULL, NULL );
-				msc->msc_ld = NULL;
+
+#ifdef DEBUG_205
+				Debug( LDAP_DEBUG_ANY,
+					"### %s meta_back_init_one_conn(rewrite) "
+					"ldap_unbind_ext[%d] ld=%p\n",
+					op->o_log_prefix, candidate,
+					(void *)msc->msc_ld );
+#endif /* DEBUG_205 */
+
+				/* need to trash a connection not fully established */
+				meta_clear_one_candidate( op, mc, candidate );
 				goto error_return;
 			}
 			
@@ -491,6 +606,8 @@ retry:;
 				ber_dupbv( &msc->msc_bound_ndn, &op->o_conn->c_dn );
 			}
 
+			assert( !BER_BVISNULL( &msc->msc_bound_ndn ) );
+
 		} else {
 			ber_dupbv( &msc->msc_bound_ndn, (struct berval *)&slap_empty_bv );
 		}
@@ -499,17 +616,25 @@ retry:;
 	assert( !BER_BVISNULL( &msc->msc_bound_ndn ) );
 
 error_return:;
+	if ( dolock ) {
+		ldap_pvt_thread_mutex_lock( &mi->mi_conninfo.lai_mutex );
+	}
+	META_BACK_CONN_CREATING_CLEAR( msc );
 	if ( rs->sr_err == LDAP_SUCCESS ) {
 		/*
 		 * Sets a cookie for the rewrite session
 		 */
 		( void )rewrite_session_init( mt->mt_rwmap.rwm_rw, op->o_conn );
+		META_BACK_CONN_INITED_SET( msc );
+	}
+	if ( dolock ) {
+		ldap_pvt_thread_mutex_unlock( &mi->mi_conninfo.lai_mutex );
+	}
 
-	} else {
+	if ( rs->sr_err != LDAP_SUCCESS ) {
 		rs->sr_err = slap_map_api2result( rs );
 		if ( sendok & LDAP_BACK_SENDERR ) {
 			send_ldap_result( op, rs );
-			rs->sr_text = NULL;
 		}
 	}
 
@@ -534,15 +659,22 @@ meta_back_retry(
 	metaconn_t		*mc = *mcp;
 	metasingleconn_t	*msc = &mc->mc_conns[ candidate ];
 	int			rc = LDAP_UNAVAILABLE,
-				binding = LDAP_BACK_CONN_BINDING( msc );
+				binding,
+				quarantine = 1;
 
 	ldap_pvt_thread_mutex_lock( &mi->mi_conninfo.lai_mutex );
 
+	assert( !META_BACK_CONN_CREATING( msc ) );
+	binding = LDAP_BACK_CONN_BINDING( msc );
+	LDAP_BACK_CONN_BINDING_CLEAR( msc );
+
 	assert( mc->mc_refcnt > 0 );
 	if ( mc->mc_refcnt == 1 ) {
 		if ( LogTest( LDAP_DEBUG_ANY ) ) {
 			char	buf[ SLAP_TEXT_BUFLEN ];
 
+			/* this lock is required; however,
+			 * it's invoked only when logging is on */
 			ldap_pvt_thread_mutex_lock( &mt->mt_uri_mutex );
 			snprintf( buf, sizeof( buf ),
 				"retrying URI=\"%s\" DN=\"%s\"",
@@ -556,19 +688,22 @@ meta_back_retry(
 				op->o_log_prefix, candidate, buf );
 		}
 
-		meta_clear_one_candidate( msc );
+		meta_clear_one_candidate( op, mc, candidate );
 		LDAP_BACK_CONN_ISBOUND_CLEAR( msc );
 
 		( void )rewrite_session_delete( mt->mt_rwmap.rwm_rw, op->o_conn );
 
 		/* mc here must be the regular mc, reset and ready for init */
 		rc = meta_back_init_one_conn( op, rs, mc, candidate,
-			LDAP_BACK_CONN_ISPRIV( mc ), sendok );
+			LDAP_BACK_CONN_ISPRIV( mc ), sendok, 0 );
+
+		/* restore the "binding" flag, in case */
 		if ( binding ) {
 			LDAP_BACK_CONN_BINDING_SET( msc );
 		}
 
 		if ( rc == LDAP_SUCCESS ) {
+			quarantine = 0;
 			rc = meta_back_single_dobind( op, rs, mcp, candidate,
 				sendok, mt->mt_nretries, 0 );
 
@@ -577,13 +712,24 @@ meta_back_retry(
 				"meta_back_single_dobind=%d\n",
 				op->o_log_prefix, candidate, rc );
 			if ( rc == LDAP_SUCCESS ) {
-				if ( be_isroot( op )  ) {
+				if ( !BER_BVISNULL( &msc->msc_bound_ndn ) &&
+					!BER_BVISEMPTY( &msc->msc_bound_ndn ) )
+				{
 					LDAP_BACK_CONN_ISBOUND_SET( msc );
+
 				} else {
 					LDAP_BACK_CONN_ISANON_SET( msc );
 				}
+
+				/* when bound, dispose of the "binding" flag */
+				if ( binding ) {
+					LDAP_BACK_CONN_BINDING_CLEAR( msc );
+				}
 			}
         	}
+
+		/* don't send twice */
+		sendok &= ~LDAP_BACK_SENDERR;
 	}
 
 	if ( rc != LDAP_SUCCESS ) {
@@ -592,27 +738,61 @@ meta_back_retry(
 		candidates[ candidate ].sr_err = rc;
 
 		if ( *mcp != NULL ) {
-			if ( binding ) {
-				LDAP_BACK_CONN_BINDING_CLEAR( msc );
+			if ( mc->mc_refcnt == 1 ) {
+				if ( binding ) {
+					LDAP_BACK_CONN_BINDING_CLEAR( msc );
+				}
+				(void)meta_clear_one_candidate( op, mc, candidate );
 			}
+
 			LDAP_BACK_CONN_TAINTED_SET( mc );
 			/* only release if mandatory; otherwise
 			 * let the caller do what's best before
 			 * releasing */
 			if ( META_BACK_ONERR_STOP( mi ) ) {
-				meta_back_release_conn_lock( op, mc, 0 );
+				meta_back_release_conn_lock( mi, mc, 0 );
 				*mcp = NULL;
+
+			} else {
+#if META_BACK_PRINT_CONNTREE > 0
+				meta_back_print_conntree( mi, ">>> meta_back_retry" );
+#endif /* META_BACK_PRINT_CONNTREE */
+
+				/* FIXME: could be done better, reworking meta_back_release_conn_lock() */
+				if ( LDAP_BACK_PCONN_ISPRIV( mc ) ) {
+					if ( mc->mc_q.tqe_prev != NULL ) {
+						assert( LDAP_BACK_CONN_CACHED( mc ) );
+						assert( mi->mi_conn_priv[ LDAP_BACK_CONN2PRIV( mc ) ].mic_num > 0 );
+						LDAP_TAILQ_REMOVE( &mi->mi_conn_priv[ LDAP_BACK_CONN2PRIV( mc ) ].mic_priv,
+							mc, mc_q );
+						mi->mi_conn_priv[ LDAP_BACK_CONN2PRIV( mc ) ].mic_num--;
+						LDAP_TAILQ_ENTRY_INIT( mc, mc_q );
+
+					} else {
+						assert( !LDAP_BACK_CONN_CACHED( mc ) );
+					}
+
+				} else {
+					/* FIXME: check if in tree, for consistency? */
+					(void)avl_delete( &mi->mi_conninfo.lai_tree,
+						( caddr_t )mc, meta_back_conndnmc_cmp );
+				}
+				LDAP_BACK_CONN_CACHED_CLEAR( mc );
+
+#if META_BACK_PRINT_CONNTREE > 0
+				meta_back_print_conntree( mi, "<<< meta_back_retry" );
+#endif /* META_BACK_PRINT_CONNTREE */
 			}
 		}
 
-		if ( sendok ) {
+		if ( sendok & LDAP_BACK_SENDERR ) {
 			rs->sr_err = rc;
-			rs->sr_text = NULL;
+			rs->sr_text = "Unable to retry";
 			send_ldap_result( op, rs );
 		}
 	}
 
-	if ( META_BACK_TGT_QUARANTINE( mt ) ) {
+	if ( quarantine && META_BACK_TGT_QUARANTINE( mt ) ) {
 		meta_back_quarantine( op, rs, candidate );
 	}
 
@@ -666,7 +846,7 @@ meta_back_get_candidate(
 	 */
 	if ( candidate == META_TARGET_NONE ) {
 		rs->sr_err = LDAP_NO_SUCH_OBJECT;
-		rs->sr_text = "no suitable candidate target found";
+		rs->sr_text = "No suitable candidate target found";
 
 	} else if ( candidate == META_TARGET_MULTIPLE ) {
 		Filter		f = { 0 };
@@ -719,7 +899,7 @@ meta_back_get_candidate(
 
 			} else {
 				rs->sr_err = LDAP_UNWILLING_TO_PERFORM;
-				rs->sr_text = "cannot select unique candidate target";
+				rs->sr_text = "Unable to select unique candidate target";
 			}
 			break;
 		}
@@ -845,59 +1025,143 @@ meta_back_getconn(
 		META_DNTYPE_ENTRY,
 		META_DNTYPE_PARENT,
 		META_DNTYPE_NEWPARENT
-			} dn_type = META_DNTYPE_ENTRY;
+	}		dn_type = META_DNTYPE_ENTRY;
 	struct berval	ndn = op->o_req_ndn,
 			pndn;
 
 	SlapReply	*candidates = meta_back_candidates_get( op );
 
 	/* Internal searches are privileged and shared. So is root. */
-	/* FIXME: there seem to be concurrency issues */
-	if ( op->o_do_not_cache || be_isroot( op ) ) {
-		mc_curr.mc_local_ndn = op->o_bd->be_rootndn;
+	if ( ( !BER_BVISEMPTY( &op->o_ndn ) && META_BACK_PROXYAUTHZ_ALWAYS( mi ) )
+		|| ( BER_BVISEMPTY( &op->o_ndn ) && META_BACK_PROXYAUTHZ_ANON( mi ) )
+		|| op->o_do_not_cache || be_isroot( op ) )
+	{
 		LDAP_BACK_CONN_ISPRIV_SET( &mc_curr );
-		mc_curr.mc_conn = LDAP_BACK_PCONN_SET( op );
+		mc_curr.mc_local_ndn = op->o_bd->be_rootndn;
+		LDAP_BACK_PCONN_ROOTDN_SET( &mc_curr, op );
+
+	} else if ( BER_BVISEMPTY( &op->o_ndn ) && META_BACK_PROXYAUTHZ_NOANON( mi ) )
+	{
+		LDAP_BACK_CONN_ISANON_SET( &mc_curr );
+		BER_BVSTR( &mc_curr.mc_local_ndn, "" );
+		LDAP_BACK_PCONN_ANON_SET( &mc_curr, op );
 
 	} else {
 		mc_curr.mc_local_ndn = op->o_ndn;
 
 		/* Explicit binds must not be shared */
-		if ( op->o_tag == LDAP_REQ_BIND || SLAP_IS_AUTHZ_BACKEND( op ) ) {
+		if ( !BER_BVISEMPTY( &op->o_ndn )
+			|| op->o_tag == LDAP_REQ_BIND
+			|| SLAP_IS_AUTHZ_BACKEND( op ) )
+		{
 			mc_curr.mc_conn = op->o_conn;
 	
 		} else {
-			mc_curr.mc_conn = LDAP_BACK_PCONN_SET( op );
+			LDAP_BACK_CONN_ISANON_SET( &mc_curr );
+			LDAP_BACK_PCONN_ANON_SET( &mc_curr, op );
 		}
 	}
 
 	/* Explicit Bind requests always get their own conn */
-	if ( !( sendok & LDAP_BACK_BINDING ) ) {
+	if ( sendok & LDAP_BACK_BINDING ) {
+		mc_curr.mc_conn = op->o_conn;
+
+	} else {
 		/* Searches for a metaconn in the avl tree */
 retry_lock:;
 		ldap_pvt_thread_mutex_lock( &mi->mi_conninfo.lai_mutex );
-		mc = (metaconn_t *)avl_find( mi->mi_conninfo.lai_tree, 
-			(caddr_t)&mc_curr, meta_back_conndn_cmp );
-		if ( mc ) {
-			if ( ( mi->mi_conn_ttl != 0 && op->o_time > mc->mc_create_time + mi->mi_conn_ttl )
-				|| ( mi->mi_idle_timeout != 0 && op->o_time > mc->mc_time + mi->mi_idle_timeout ) )
+		if ( LDAP_BACK_PCONN_ISPRIV( &mc_curr ) ) {
+			/* lookup a conn that's not binding */
+			LDAP_TAILQ_FOREACH( mc,
+				&mi->mi_conn_priv[ LDAP_BACK_CONN2PRIV( &mc_curr ) ].mic_priv,
+				mc_q )
 			{
-				/* don't let anyone else use this expired connection */
-				(void)avl_delete( &mi->mi_conninfo.lai_tree,
-					(caddr_t)mc, meta_back_conndnmc_cmp );
-				LDAP_BACK_CONN_TAINTED_SET( mc );
-
-				Debug( LDAP_DEBUG_TRACE, "%s meta_back_getconn: mc=%p conn=%ld expired.\n",
-					op->o_log_prefix, (void *)mc, LDAP_BACK_PCONN_ID( mc->mc_conn ) );
+				if ( !LDAP_BACK_CONN_BINDING( mc ) && mc->mc_refcnt == 0 ) {
+					break;
+				}
 			}
 
-			/* Don't reuse connections while they're still binding */
+			if ( mc != NULL ) {
+				if ( mc != LDAP_TAILQ_LAST( &mi->mi_conn_priv[ LDAP_BACK_CONN2PRIV( mc ) ].mic_priv,
+					metaconn_t, mc_q ) )
+				{
+					LDAP_TAILQ_REMOVE( &mi->mi_conn_priv[ LDAP_BACK_CONN2PRIV( mc ) ].mic_priv,
+						mc, mc_q );
+					LDAP_TAILQ_ENTRY_INIT( mc, mc_q );
+					LDAP_TAILQ_INSERT_TAIL( &mi->mi_conn_priv[ LDAP_BACK_CONN2PRIV( mc ) ].mic_priv,
+						mc, mc_q );
+				}
+
+			} else if ( !LDAP_BACK_USE_TEMPORARIES( mi )
+				&& mi->mi_conn_priv[ LDAP_BACK_CONN2PRIV( &mc_curr ) ].mic_num == mi->mi_conn_priv_max )
+			{
+				mc = LDAP_TAILQ_FIRST( &mi->mi_conn_priv[ LDAP_BACK_CONN2PRIV( &mc_curr ) ].mic_priv );
+			}
+			
+
+		} else {
+			mc = (metaconn_t *)avl_find( mi->mi_conninfo.lai_tree, 
+				(caddr_t)&mc_curr, meta_back_conndn_cmp );
+		}
+
+		if ( mc ) {
+			/* catch taint errors */
+			assert( !LDAP_BACK_CONN_TAINTED( mc ) );
+
+			/* Don't reuse connections while they're still binding
+			 * NOTE: only makes sense for binds */
 			if ( LDAP_BACK_CONN_BINDING( mc ) ) {
-				ldap_pvt_thread_mutex_unlock( &mi->mi_conninfo.lai_mutex );
-				ldap_pvt_thread_yield();
-				goto retry_lock;
-			}
+				if ( !LDAP_BACK_USE_TEMPORARIES( mi ) ) {
+					ldap_pvt_thread_mutex_unlock( &mi->mi_conninfo.lai_mutex );
 
-			mc->mc_refcnt++;
+					ldap_pvt_thread_yield();
+					goto retry_lock;
+				}
+
+				/* release conn, and create a temporary */
+				mc = NULL;
+
+			} else {
+				if ( ( mi->mi_conn_ttl != 0 && op->o_time > mc->mc_create_time + mi->mi_conn_ttl )
+					|| ( mi->mi_idle_timeout != 0 && op->o_time > mc->mc_time + mi->mi_idle_timeout ) )
+				{
+#if META_BACK_PRINT_CONNTREE > 0
+					meta_back_print_conntree( mi,
+						">>> meta_back_getconn(expired)" );
+#endif /* META_BACK_PRINT_CONNTREE */
+
+					/* don't let anyone else use this expired connection */
+					if ( LDAP_BACK_PCONN_ISPRIV( mc ) ) {
+						if ( mc->mc_q.tqe_prev != NULL ) {
+							assert( LDAP_BACK_CONN_CACHED( mc ) );
+							assert( mi->mi_conn_priv[ LDAP_BACK_CONN2PRIV( mc ) ].mic_num > 0 );
+							LDAP_TAILQ_REMOVE( &mi->mi_conn_priv[ LDAP_BACK_CONN2PRIV( mc ) ].mic_priv,
+								mc, mc_q );
+							mi->mi_conn_priv[ LDAP_BACK_CONN2PRIV( mc ) ].mic_num--;
+							LDAP_TAILQ_ENTRY_INIT( mc, mc_q );
+
+						} else {
+							assert( !LDAP_BACK_CONN_CACHED( mc ) );
+						}
+
+					} else {
+						(void)avl_delete( &mi->mi_conninfo.lai_tree,
+							(caddr_t)mc, meta_back_conndnmc_cmp );
+					}
+
+#if META_BACK_PRINT_CONNTREE > 0
+					meta_back_print_conntree( mi,
+						"<<< meta_back_getconn(expired)" );
+#endif /* META_BACK_PRINT_CONNTREE */
+					LDAP_BACK_CONN_TAINTED_SET( mc );
+					LDAP_BACK_CONN_CACHED_CLEAR( mc );
+
+					Debug( LDAP_DEBUG_TRACE, "%s meta_back_getconn: mc=%p conn=%ld expired (tainted).\n",
+						op->o_log_prefix, (void *)mc, LDAP_BACK_PCONN_ID( mc ) );
+				}
+
+				mc->mc_refcnt++;
+			}
 		}
 		ldap_pvt_thread_mutex_unlock( &mi->mi_conninfo.lai_mutex );
 	}
@@ -926,12 +1190,12 @@ retry_lock:;
 		}
 		break;
 
+	case LDAP_REQ_COMPARE:
 	case LDAP_REQ_DELETE:
 	case LDAP_REQ_MODIFY:
 		/* just a unique candidate */
 		break;
 
-	case LDAP_REQ_COMPARE:
 	case LDAP_REQ_SEARCH:
 		/* allow multiple candidates for the searchBase */
 		op_type = META_OP_ALLOW_MULTIPLE;
@@ -957,6 +1221,16 @@ retry_lock:;
 			if ( sendok & LDAP_BACK_BINDING ) {
 				LDAP_BACK_CONN_BINDING_SET( mc );
 			}
+			if ( LDAP_BACK_CONN_ISPRIV( &mc_curr ) ) {
+				LDAP_BACK_CONN_ISPRIV_SET( mc );
+
+			} else if ( LDAP_BACK_CONN_ISANON( &mc_curr ) ) {
+				LDAP_BACK_CONN_ISANON_SET( mc );
+			}
+
+		} else if ( 0 ) {
+			/* TODO: if any of the connections is binding,
+			 * release mc and create a new one */
 		}
 
 		for ( i = 0; i < mi->mi_ntargets; i++ ) {
@@ -966,7 +1240,7 @@ retry_lock:;
 			 */
 			candidates[ i ].sr_err = meta_back_init_one_conn( op,
 				rs, mc, i, LDAP_BACK_CONN_ISPRIV( &mc_curr ),
-				sendok );
+				LDAP_BACK_DONTSEND, !new_conn );
 			if ( candidates[ i ].sr_err == LDAP_SUCCESS ) {
 				META_CANDIDATE_SET( &candidates[ i ] );
 				ncandidates++;
@@ -990,7 +1264,7 @@ retry_lock:;
 				meta_back_conn_free( mc );
 
 			} else {
-				meta_back_release_conn( op, mc );
+				meta_back_release_conn( mi, mc );
 			}
 
 			rs->sr_err = LDAP_NO_SUCH_OBJECT;
@@ -1001,7 +1275,6 @@ retry_lock:;
 					rs->sr_matched = op->o_bd->be_suffix[ 0 ].bv_val;
 				}
 				send_ldap_result( op, rs );
-				rs->sr_text = NULL;
 				rs->sr_matched = NULL;
 			}
 
@@ -1041,7 +1314,7 @@ retry_lock:;
 	
 			if ( i < 0 || rs->sr_err != LDAP_SUCCESS ) {
 				if ( mc != NULL ) {
-					meta_back_release_conn( op, mc );
+					meta_back_release_conn( mi, mc );
 				}
 
 				if ( sendok & LDAP_BACK_SENDERR ) {
@@ -1049,7 +1322,6 @@ retry_lock:;
 						rs->sr_matched = op->o_bd->be_suffix[ 0 ].bv_val;
 					}
 					send_ldap_result( op, rs );
-					rs->sr_text = NULL;
 					rs->sr_matched = NULL;
 				}
 			
@@ -1060,14 +1332,13 @@ retry_lock:;
 		if ( dn_type == META_DNTYPE_NEWPARENT && meta_back_get_candidate( op, rs, op->orr_nnewSup ) != i )
 		{
 			if ( mc != NULL ) {
-				meta_back_release_conn( op, mc );
+				meta_back_release_conn( mi, mc );
 			}
 
 			rs->sr_err = LDAP_UNWILLING_TO_PERFORM;
-			rs->sr_text = "cross-target rename not supported";
+			rs->sr_text = "Cross-target rename not supported";
 			if ( sendok & LDAP_BACK_SENDERR ) {
 				send_ldap_result( op, rs );
-				rs->sr_text = NULL;
 			}
 
 			return NULL;
@@ -1087,14 +1358,24 @@ retry_lock2:;
 				mc = (metaconn_t *)avl_find( mi->mi_conninfo.lai_tree, 
 					(caddr_t)&mc_curr, meta_back_conndn_cmp );
 				if ( mc != NULL ) {
-					/* Don't reuse connections while they're still binding */
-					if ( LDAP_BACK_CONN_BINDING( mc ) ) {
-						ldap_pvt_thread_mutex_unlock( &mi->mi_conninfo.lai_mutex );
-						ldap_pvt_thread_yield();
-						goto retry_lock2;
-					}
+					/* catch taint errors */
+					assert( !LDAP_BACK_CONN_TAINTED( mc ) );
 
-					mc->mc_refcnt++;
+					/* Don't reuse connections while they're still binding */
+					if ( META_BACK_CONN_CREATING( &mc->mc_conns[ i ] )
+						|| LDAP_BACK_CONN_BINDING( &mc->mc_conns[ i ] ) )
+					{
+						if ( !LDAP_BACK_USE_TEMPORARIES( mi ) ) {
+							ldap_pvt_thread_mutex_unlock( &mi->mi_conninfo.lai_mutex );
+							ldap_pvt_thread_yield();
+							goto retry_lock2;
+						}
+
+						mc = NULL;
+
+					} else {
+						mc->mc_refcnt++;
+					}
 				}
 				ldap_pvt_thread_mutex_unlock( &mi->mi_conninfo.lai_mutex );
 			}
@@ -1109,6 +1390,12 @@ retry_lock2:;
 				if ( sendok & LDAP_BACK_BINDING ) {
 					LDAP_BACK_CONN_BINDING_SET( mc );
 				}
+				if ( LDAP_BACK_CONN_ISPRIV( &mc_curr ) ) {
+					LDAP_BACK_CONN_ISPRIV_SET( mc );
+
+				} else if ( LDAP_BACK_CONN_ISANON( &mc_curr ) ) {
+					LDAP_BACK_CONN_ISANON_SET( mc );
+				}
 			}
 		}
 
@@ -1126,7 +1413,7 @@ retry_lock2:;
 		 * sends the appropriate result.
 		 */
 		err = meta_back_init_one_conn( op, rs, mc, i,
-			LDAP_BACK_CONN_ISPRIV( &mc_curr ), sendok );
+			LDAP_BACK_CONN_ISPRIV( &mc_curr ), sendok, !new_conn );
 		if ( err != LDAP_SUCCESS ) {
 			/*
 			 * FIXME: in case one target cannot
@@ -1139,7 +1426,7 @@ retry_lock2:;
 				meta_back_conn_free( mc );
 
 			} else {
-				meta_back_release_conn( op, mc );
+				meta_back_release_conn( mi, mc );
 			}
 			return NULL;
 		}
@@ -1164,14 +1451,18 @@ retry_lock2:;
 			mc->mc_conn = mc_curr.mc_conn;
 			ber_dupbv( &mc->mc_local_ndn, &mc_curr.mc_local_ndn );
 			new_conn = 1;
-			if ( sendok & LDAP_BACK_BINDING ) {
-				LDAP_BACK_CONN_BINDING_SET( mc );
+			if ( LDAP_BACK_CONN_ISPRIV( &mc_curr ) ) {
+				LDAP_BACK_CONN_ISPRIV_SET( mc );
+
+			} else if ( LDAP_BACK_CONN_ISANON( &mc_curr ) ) {
+				LDAP_BACK_CONN_ISANON_SET( mc );
 			}
 		}
 
 		for ( i = 0; i < mi->mi_ntargets; i++ ) {
 			metatarget_t		*mt = mi->mi_targets[ i ];
-			metasingleconn_t	*msc = &mc->mc_conns[ i ];
+
+			META_CANDIDATE_RESET( &candidates[ i ] );
 
 			if ( i == cached 
 				|| meta_back_is_candidate( mt, &op->o_req_ndn,
@@ -1183,10 +1474,11 @@ retry_lock2:;
 				 * also init'd
 				 */
 				int lerr = meta_back_init_one_conn( op, rs, mc, i,
-					LDAP_BACK_CONN_ISPRIV( &mc_curr ), LDAP_BACK_DONTSEND );
+					LDAP_BACK_CONN_ISPRIV( &mc_curr ),
+					LDAP_BACK_DONTSEND, !new_conn );
+				candidates[ i ].sr_err = lerr;
 				if ( lerr == LDAP_SUCCESS ) {
 					META_CANDIDATE_SET( &candidates[ i ] );
-					candidates[ i ].sr_err = LDAP_SUCCESS;
 					ncandidates++;
 
 					Debug( LDAP_DEBUG_TRACE, "%s: meta_back_getconn[%d]\n",
@@ -1194,7 +1486,6 @@ retry_lock2:;
 
 				} else if ( lerr == LDAP_UNAVAILABLE && !META_BACK_ONERR_STOP( mi ) ) {
 					META_CANDIDATE_SET( &candidates[ i ] );
-					candidates[ i ].sr_err = LDAP_UNAVAILABLE;
 
 					Debug( LDAP_DEBUG_TRACE, "%s: meta_back_getconn[%d] %s\n",
 						op->o_log_prefix, i,
@@ -1208,46 +1499,42 @@ retry_lock2:;
 					 * be tried?
 					 */
 					if ( new_conn ) {
-						( void )meta_clear_one_candidate( msc );
+						( void )meta_clear_one_candidate( op, mc, i );
 					}
 					/* leave the target candidate, but record the error for later use */
-					candidates[ i ].sr_err = lerr;
 					err = lerr;
 
 					if ( lerr == LDAP_UNAVAILABLE && mt->mt_isquarantined != LDAP_BACK_FQ_NO ) {
-						Debug( LDAP_DEBUG_TRACE, "%s: meta_back_getconn[%d] quarantined: %d\n",
+						Debug( LDAP_DEBUG_TRACE, "%s: meta_back_getconn[%d] quarantined err=%d\n",
 							op->o_log_prefix, i, lerr );
 
 					} else {
-						Debug( LDAP_DEBUG_ANY, "%s: meta_back_getconn[%d] failed: %d\n",
+						Debug( LDAP_DEBUG_ANY, "%s: meta_back_getconn[%d] failed err=%d\n",
 							op->o_log_prefix, i, lerr );
 					}
 
 					if ( META_BACK_ONERR_STOP( mi ) ) {
 						if ( sendok & LDAP_BACK_SENDERR ) {
 							send_ldap_result( op, rs );
-							rs->sr_text = NULL;
 						}
 						if ( new_conn ) {
 							mc->mc_refcnt = 0;
 							meta_back_conn_free( mc );
 
 						} else {
-							meta_back_release_conn( op, mc );
+							meta_back_release_conn( mi, mc );
 						}
 
 						return NULL;
 					}
 
-					rs->sr_text = NULL;
 					continue;
 				}
 
 			} else {
 				if ( new_conn ) {
-					( void )meta_clear_one_candidate( msc );
+					( void )meta_clear_one_candidate( op, mc, i );
 				}
-				META_CANDIDATE_RESET( &candidates[ i ] );
 			}
 		}
 
@@ -1257,18 +1544,19 @@ retry_lock2:;
 				meta_back_conn_free( mc );
 
 			} else {
-				meta_back_release_conn( op, mc );
+				meta_back_release_conn( mi, mc );
 			}
 
-			rs->sr_err = LDAP_NO_SUCH_OBJECT;
-			rs->sr_text = "Unable to select valid candidates";
+			if ( rs->sr_err == LDAP_SUCCESS ) {
+				rs->sr_err = LDAP_NO_SUCH_OBJECT;
+				rs->sr_text = "Unable to select valid candidates";
+			}
 
 			if ( sendok & LDAP_BACK_SENDERR ) {
 				if ( rs->sr_err == LDAP_NO_SUCH_OBJECT ) {
 					rs->sr_matched = op->o_bd->be_suffix[ 0 ].bv_val;
 				}
 				send_ldap_result( op, rs );
-				rs->sr_text = NULL;
 				rs->sr_matched = NULL;
 			}
 
@@ -1295,72 +1583,94 @@ done:;
 		 * Inserts the newly created metaconn in the avl tree
 		 */
 		ldap_pvt_thread_mutex_lock( &mi->mi_conninfo.lai_mutex );
-		err = avl_insert( &mi->mi_conninfo.lai_tree, ( caddr_t )mc,
-			       	meta_back_conndn_cmp, meta_back_conndn_dup );
+#if META_BACK_PRINT_CONNTREE > 0
+		meta_back_print_conntree( mi, ">>> meta_back_getconn" );
+#endif /* META_BACK_PRINT_CONNTREE */
 
-#if PRINT_CONNTREE > 0
-		myprint( mi->mi_conninfo.lai_tree, "meta_back_getconn" );
-#endif /* PRINT_CONNTREE */
-		
+		if ( LDAP_BACK_PCONN_ISPRIV( mc ) ) {
+			if ( mi->mi_conn_priv[ LDAP_BACK_CONN2PRIV( mc ) ].mic_num < mi->mi_conn_priv_max ) {
+				LDAP_TAILQ_INSERT_TAIL( &mi->mi_conn_priv[ LDAP_BACK_CONN2PRIV( mc ) ].mic_priv, mc, mc_q );
+				mi->mi_conn_priv[ LDAP_BACK_CONN2PRIV( mc ) ].mic_num++;
+				LDAP_BACK_CONN_CACHED_SET( mc );
+
+			} else {
+				LDAP_BACK_CONN_TAINTED_SET( mc );
+			}
+			rs->sr_err = 0;
+
+		} else {
+			err = avl_insert( &mi->mi_conninfo.lai_tree, ( caddr_t )mc,
+			       	meta_back_conndn_cmp, meta_back_conndn_dup );
+			LDAP_BACK_CONN_CACHED_SET( mc );
+		}
+
+#if META_BACK_PRINT_CONNTREE > 0
+		meta_back_print_conntree( mi, ">>> meta_back_getconn" );
+#endif /* META_BACK_PRINT_CONNTREE */
 		ldap_pvt_thread_mutex_unlock( &mi->mi_conninfo.lai_mutex );
 
-		/*
-		 * Err could be -1 in case a duplicate metaconn is inserted
-		 */
-		switch ( err ) {
-		case 0:
-			break;
+		if ( !LDAP_BACK_PCONN_ISPRIV( mc ) ) {
+			/*
+			 * Err could be -1 in case a duplicate metaconn is inserted
+			 */
+			switch ( err ) {
+			case 0:
+				break;
 
-		case -1:
-			/* duplicate: free and try to get the newly created one */
-			if ( !( sendok & LDAP_BACK_BINDING ) ) {
-				new_conn = 0;
-				goto retry_lock;
-			}
-			LDAP_BACK_CONN_TAINTED_SET( mc );
-			break;
-
-		default:
-			Debug( LDAP_DEBUG_ANY,
-				"%s meta_back_getconn: candidates=%d conn=%ld insert failed\n",
-				op->o_log_prefix, ncandidates,
-				LDAP_BACK_PCONN_ID( mc->mc_conn ) );
+			case -1:
+				LDAP_BACK_CONN_CACHED_CLEAR( mc );
+				/* duplicate: free and try to get the newly created one */
+				if ( !( sendok & LDAP_BACK_BINDING ) && !LDAP_BACK_USE_TEMPORARIES( mi ) ) {
+					mc->mc_refcnt = 0;	
+					meta_back_conn_free( mc );
 	
-			mc->mc_refcnt = 0;	
-			meta_back_conn_free( mc );
+					new_conn = 0;
+					goto retry_lock;
+				}
 
-			rs->sr_err = LDAP_OTHER;
-			rs->sr_text = "proxy bind collision";
-			if ( sendok & LDAP_BACK_SENDERR ) {
-				send_ldap_result( op, rs );
-				rs->sr_text = NULL;
+				LDAP_BACK_CONN_TAINTED_SET( mc );
+				break;
+
+			default:
+				LDAP_BACK_CONN_CACHED_CLEAR( mc );
+				Debug( LDAP_DEBUG_ANY,
+					"%s meta_back_getconn: candidates=%d conn=%ld insert failed\n",
+					op->o_log_prefix, ncandidates,
+					LDAP_BACK_PCONN_ID( mc ) );
+	
+				mc->mc_refcnt = 0;	
+				meta_back_conn_free( mc );
+
+				rs->sr_err = LDAP_OTHER;
+				rs->sr_text = "Proxy bind collision";
+				if ( sendok & LDAP_BACK_SENDERR ) {
+					send_ldap_result( op, rs );
+				}
+				return NULL;
 			}
-			return NULL;
 		}
 
 		Debug( LDAP_DEBUG_TRACE,
 			"%s meta_back_getconn: candidates=%d conn=%ld inserted\n",
 			op->o_log_prefix, ncandidates,
-			LDAP_BACK_PCONN_ID( mc->mc_conn ) );
+			LDAP_BACK_PCONN_ID( mc ) );
 
 	} else {
 		Debug( LDAP_DEBUG_TRACE,
 			"%s meta_back_getconn: candidates=%d conn=%ld fetched\n",
 			op->o_log_prefix, ncandidates,
-			LDAP_BACK_PCONN_ID( mc->mc_conn ) );
+			LDAP_BACK_PCONN_ID( mc ) );
 	}
-	
+
 	return mc;
 }
 
 void
 meta_back_release_conn_lock(
-       	Operation 		*op,
+       	metainfo_t		*mi,
 	metaconn_t		*mc,
 	int			dolock )
 {
-	metainfo_t	*mi = ( metainfo_t * )op->o_bd->be_private;
-
 	assert( mc != NULL );
 
 	if ( dolock ) {
@@ -1368,16 +1678,55 @@ meta_back_release_conn_lock(
 	}
 	assert( mc->mc_refcnt > 0 );
 	mc->mc_refcnt--;
-	LDAP_BACK_CONN_BINDING_CLEAR( mc );
+	/* NOTE: the connection is removed if either it is tainted
+	 * or if it is shared and no one else is using it.  This needs
+	 * to occur because for intrinsic reasons cached connections
+	 * that are not privileged would live forever and pollute
+	 * the connection space (and eat up resources).  Maybe this
+	 * should be configurable... */
 	if ( LDAP_BACK_CONN_TAINTED( mc ) ) {
-		Debug( LDAP_DEBUG_TRACE, "%s meta_back_release_conn: mc=%p conn=%ld tainted.\n",
-			op->o_log_prefix, (void *)mc, LDAP_BACK_PCONN_ID( mc->mc_conn ) );
-		(void)avl_delete( &mi->mi_conninfo.lai_tree,
-			( caddr_t )mc, meta_back_conndnmc_cmp );
+#if META_BACK_PRINT_CONNTREE > 0
+		meta_back_print_conntree( mi, ">>> meta_back_release_conn" );
+#endif /* META_BACK_PRINT_CONNTREE */
+
+		if ( LDAP_BACK_PCONN_ISPRIV( mc ) ) {
+			if ( mc->mc_q.tqe_prev != NULL ) {
+				assert( LDAP_BACK_CONN_CACHED( mc ) );
+				assert( mi->mi_conn_priv[ LDAP_BACK_CONN2PRIV( mc ) ].mic_num > 0 );
+				LDAP_TAILQ_REMOVE( &mi->mi_conn_priv[ LDAP_BACK_CONN2PRIV( mc ) ].mic_priv, mc, mc_q );
+				mi->mi_conn_priv[ LDAP_BACK_CONN2PRIV( mc ) ].mic_num--;
+				LDAP_TAILQ_ENTRY_INIT( mc, mc_q );
+
+			} else {
+				assert( !LDAP_BACK_CONN_CACHED( mc ) );
+			}
+
+		} else {
+			metaconn_t	*tmpmc;
+
+			tmpmc = avl_delete( &mi->mi_conninfo.lai_tree,
+				( caddr_t )mc, meta_back_conndnmc_cmp );
+
+			/* Overparanoid, but useful... */
+			assert( tmpmc == NULL || tmpmc == mc );
+		}
+
+		LDAP_BACK_CONN_CACHED_CLEAR( mc );
+
+#if META_BACK_PRINT_CONNTREE > 0
+		meta_back_print_conntree( mi, "<<< meta_back_release_conn" );
+#endif /* META_BACK_PRINT_CONNTREE */
+
 		if ( mc->mc_refcnt == 0 ) {
 			meta_back_conn_free( mc );
+			mc = NULL;
 		}
 	}
+
+	if ( mc != NULL && LDAP_BACK_CONN_BINDING( mc ) ) {
+		LDAP_BACK_CONN_BINDING_CLEAR( mc );
+	}
+
 	if ( dolock ) {
 		ldap_pvt_thread_mutex_unlock( &mi->mi_conninfo.lai_mutex );
 	}
