@@ -1,8 +1,8 @@
 /* entry.c - monitor backend entry handling routines */
-/* $OpenLDAP: pkg/ldap/servers/slapd/back-monitor/entry.c,v 1.21.2.1 2006/08/17 23:50:31 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/back-monitor/entry.c,v 1.21.2.3 2007/02/13 20:22:26 kurt Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 2001-2006 The OpenLDAP Foundation.
+ * Copyright 2001-2007 The OpenLDAP Foundation.
  * Portions Copyright 2001-2003 Pierangelo Masarati.
  * All rights reserved.
  *
@@ -193,7 +193,7 @@ monitor_entry_stub(
 	if ( rc )
 		return NULL;
 
-	e = ch_calloc( 1, sizeof( Entry ));
+	e = entry_alloc();
 	if ( e ) {
 		struct berval nrdn;
 
