@@ -1,7 +1,7 @@
-/* $OpenLDAP: pkg/ldap/servers/slapd/modify.c,v 1.276.2.1 2006/04/18 17:33:08 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/modify.c,v 1.276.2.3 2007/02/13 20:22:18 kurt Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 1998-2006 The OpenLDAP Foundation.
+ * Copyright 1998-2007 The OpenLDAP Foundation.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -44,9 +44,6 @@ do_modify(
 	char		*last;
 	ber_tag_t	tag;
 	ber_len_t	len;
-	Modifications	*modlist = NULL;
-	Modifications	**modtail = &modlist;
-	int		increment = 0;
 	char		textbuf[ SLAP_TEXT_BUFLEN ];
 	size_t		textlen = sizeof( textbuf );
 
@@ -80,94 +77,12 @@ do_modify(
 
 	Debug( LDAP_DEBUG_ARGS, "do_modify: dn (%s)\n", dn.bv_val, 0, 0 );
 
-	/* collect modifications & save for later */
-	for ( tag = ber_first_element( op->o_ber, &len, &last );
-	    tag != LBER_DEFAULT;
-	    tag = ber_next_element( op->o_ber, &len, last ) )
-	{
-		ber_int_t mop;
-		Modifications tmp, *mod;
-
-		tmp.sml_nvalues = NULL;
-
-		if ( ber_scanf( op->o_ber, "{e{m[W]}}", &mop,
-		    &tmp.sml_type, &tmp.sml_values ) == LBER_ERROR )
-		{
-			send_ldap_discon( op, rs, LDAP_PROTOCOL_ERROR,
-				"decoding modlist error" );
-			rs->sr_err = SLAPD_DISCONNECT;
-			goto cleanup;
-		}
-
-		mod = (Modifications *) ch_malloc( sizeof(Modifications) );
-		mod->sml_op = mop;
-		mod->sml_flags = 0;
-		mod->sml_type = tmp.sml_type;
-		mod->sml_values = tmp.sml_values;
-		mod->sml_nvalues = NULL;
-		mod->sml_desc = NULL;
-		mod->sml_next = NULL;
-		*modtail = mod;
-
-		switch( mop ) {
-		case LDAP_MOD_ADD:
-			if ( mod->sml_values == NULL ) {
-				Debug( LDAP_DEBUG_ANY,
-					"do_modify: modify/add operation (%ld) requires values\n",
-					(long) mop, 0, 0 );
-
-				send_ldap_error( op, rs, LDAP_PROTOCOL_ERROR,
-					"modify/add operation requires values" );
-				goto cleanup;
-			}
-
-			/* fall through */
-
-		case LDAP_MOD_DELETE:
-		case LDAP_MOD_REPLACE:
-			break;
-
-		case LDAP_MOD_INCREMENT:
-			if( op->o_protocol >= LDAP_VERSION3 ) {
-				increment++;
-				if ( mod->sml_values == NULL ) {
-					Debug( LDAP_DEBUG_ANY, "do_modify: "
-						"modify/increment operation (%ld) requires value\n",
-						(long) mop, 0, 0 );
-
-					send_ldap_error( op, rs, LDAP_PROTOCOL_ERROR,
-						"modify/increment operation requires value" );
-					goto cleanup;
-				}
-
-				if ( !BER_BVISNULL( &mod->sml_values[ 1 ] ) ) {
-					Debug( LDAP_DEBUG_ANY, "do_modify: modify/increment "
-						"operation (%ld) requires single value\n",
-						(long) mop, 0, 0 );
-
-					send_ldap_error( op, rs, LDAP_PROTOCOL_ERROR,
-						"modify/increment operation requires single value" );
-					goto cleanup;
-				}
-
-				break;
-			}
-			/* fall thru */
-
-		default: {
-				Debug( LDAP_DEBUG_ANY,
-					"do_modify: unrecognized modify operation (%ld)\n",
-					(long) mop, 0, 0 );
-
-				send_ldap_error( op, rs, LDAP_PROTOCOL_ERROR,
-					"unrecognized modify operation" );
-				goto cleanup;
-			}
-		}
-
-		modtail = &mod->sml_next;
+	rs->sr_err = slap_parse_modlist( op, rs, op->o_ber, &op->oq_modify );
+	if ( rs->sr_err != LDAP_SUCCESS ) {
+		Debug( LDAP_DEBUG_ANY, "do_modify: slap_parse_modlist failed err=%d msg=%s\n",
+			rs->sr_err, rs->sr_text, 0 );
+		goto cleanup;
 	}
-	*modtail = NULL;
 
 	if( get_ctrls( op, rs, 1 ) != LDAP_SUCCESS ) {
 		Debug( LDAP_DEBUG_ANY, "do_modify: get_ctrls failed\n", 0, 0, 0 );
@@ -183,7 +98,7 @@ do_modify(
 		goto cleanup;
 	}
 
-	rs->sr_err = slap_mods_check( modlist,
+	rs->sr_err = slap_mods_check( op, op->orm_modlist,
 		&rs->sr_text, textbuf, textlen, NULL );
 
 	if ( rs->sr_err != LDAP_SUCCESS ) {
@@ -191,10 +106,6 @@ do_modify(
 		goto cleanup;
 	}
 
-	/* FIXME: needs review */
-	op->orm_modlist = modlist;
-	op->orm_increment = increment;
-
 	op->o_bd = frontendDB;
 	rs->sr_err = frontendDB->be_modify( op, rs );
 
@@ -442,7 +353,7 @@ slap_mods_obsolete_check(
 	char *textbuf,
 	size_t textlen )
 {
-	if( get_manageDIT( op ) ) return LDAP_SUCCESS;
+	if( get_relax( op ) ) return LDAP_SUCCESS;
 
 	for ( ; ml != NULL; ml = ml->sml_next ) {
 		if ( is_at_obsolete( ml->sml_desc->ad_type ) &&
@@ -481,7 +392,7 @@ slap_mods_no_user_mod_check(
 			continue;
 		}
 
-		if ( get_manageDIT( op ) ) {
+		if ( get_relax( op ) ) {
 			if ( ml->sml_desc->ad_type->sat_flags & SLAP_AT_MANAGEABLE ) {
 				ml->sml_flags |= SLAP_MOD_MANAGING;
 				continue;
@@ -539,6 +450,7 @@ slap_mods_no_repl_user_mod_check(
  * Do basic attribute type checking and syntax validation.
  */
 int slap_mods_check(
+	Operation *op,
 	Modifications *ml,
 	const char **text,
 	char *textbuf,
@@ -553,6 +465,12 @@ int slap_mods_check(
 		/* convert to attribute description */
 		if ( ml->sml_desc == NULL ) {
 			rc = slap_bv2ad( &ml->sml_type, &ml->sml_desc, text );
+			if( rc != LDAP_SUCCESS ) {
+				if ( get_no_schema_check( op )) {
+					rc = slap_bv2undef_ad( &ml->sml_type, &ml->sml_desc,
+						text, 0 );
+				}
+			}
 			if( rc != LDAP_SUCCESS ) {
 				snprintf( textbuf, textlen, "%s: %s",
 					ml->sml_type.bv_val, *text );
@@ -729,6 +647,8 @@ int slap_mods_check(
 			/* check for duplicates, but ignore Deletes.
 			 */
 			if( nvals > 1 && ml->sml_op != LDAP_MOD_DELETE ) {
+#define	SLAP_MODS_CHECK_QUICKSORT
+#ifndef SLAP_MODS_CHECK_QUICKSORT
 				int		i, j, rc, match;
 				MatchingRule *mr = ad->ad_type->sat_equality;
 
@@ -760,8 +680,164 @@ int slap_mods_check(
 						}
 					}
 				}
-			}
+#else	/* SLAP_MODS_CHECK_QUICKSORT */
 
+/* Quicksort + Insertion sort for small arrays */
+
+#define SMALL	8
+#define	SWAP(a,b,tmp)	tmp=(a);(a)=(b);(b)=tmp
+#define	COMP(a,b)	match=0; rc = ordered_value_match( &match, \
+						ml->sml_desc, mr, SLAP_MR_EQUALITY \
+								| SLAP_MR_VALUE_OF_ATTRIBUTE_SYNTAX \
+								| SLAP_MR_ASSERTED_VALUE_NORMALIZED_MATCH \
+								| SLAP_MR_ATTRIBUTE_VALUE_NORMALIZED_MATCH, \
+								&(a), &(b), text );
+
+					MatchingRule *mr = ad->ad_type->sat_equality;
+					int istack[sizeof(int)*16];
+					int i,j,k,l,ir,jstack, rc, match, *ix, itmp;
+					struct berval a, *cv;
+
+/* If PRESERVE_ORDER is defined only the index array is sorted; the
+ * actual values are left in their incoming order. Otherwise, the
+ * only reason to keep the index array is to identify the offending
+ * value when duplicates are found.
+ */
+#define	PRESERVE_ORDER
+#ifndef PRESERVE_ORDER
+					struct berval va, *v, *nv, bvtmp;
+
+#define	IX(x)	x
+#define	EXCH(x,y)	SWAP(ix[x],ix[y],itmp); SWAP(cv[x],cv[y],bvtmp); \
+	if (nv) {SWAP(v[x],v[y],bvtmp);}
+#define	SETA(x)	itmp = ix[x]; a = cv[x]; if (nv) va=v[x]
+#define	GETA(x)	ix[x] = itmp; cv[x] = a; if (nv) v[x]=va
+#define	SET(x,y)	ix[x] = ix[y]; cv[x] = cv[y]; if (nv) v[x]=v[y]
+
+					v = ml->sml_values;
+					nv = ml->sml_nvalues;
+
+#else	/* PRESERVE_ORDER */
+
+#define	IX(x)	ix[x]
+#define	EXCH(x,y)	SWAP(ix[x],ix[y],itmp)
+#define	SETA(x)	itmp = ix[x]; a = cv[itmp]
+#define	GETA(x)	ix[x] = itmp;
+#define	SET(x,y)	ix[x] = ix[y]
+
+#endif	/* PRESERVE_ORDER */
+
+					cv = ml->sml_nvalues ? ml->sml_nvalues : ml->sml_values;
+					if ( ad == slap_schema.si_ad_objectClass )
+						mr = NULL;	/* shortcut matching */
+
+					/* record indices to preserve input ordering */
+					ix = slap_sl_malloc( nvals * sizeof(int), ctx );
+					for (i=0; i<nvals; i++) ix[i] = i;
+
+					ir = nvals-1;
+					l = 0;
+					jstack = 0;
+
+					for(;;) {
+						if (ir - l < SMALL) {	/* Insertion sort */
+							match=1;
+							for (j=l+1;j<=ir;j++) {
+								SETA(j);
+								for (i=j-1;i>=0;i--) {
+									COMP(cv[IX(i)], a);
+									if ( match <= 0 )
+										break;
+									SET(i+1,i);
+								}
+								GETA(i+1);
+								if ( match == 0 ) goto done;
+							}
+							if ( jstack == 0 ) break;
+							if ( match == 0 ) break;
+							ir = istack[jstack--];
+							l = istack[jstack--];
+						} else {
+							k = (l + ir) >> 1;	/* Choose median of left, center, right */
+							EXCH(k, l+1);
+							COMP( cv[IX(l)], cv[IX(ir)] );
+							if ( match > 0 ) {
+								EXCH(l, ir);
+							} else if ( match == 0 ) {
+								i = ir;
+								break;
+							}
+							COMP( cv[IX(l+1)], cv[IX(ir)] );
+							if ( match > 0 ) {
+								EXCH(l+1, ir);
+							} else if ( match == 0 ) {
+								i = ir;
+								break;
+							}
+							COMP( cv[IX(l)], cv[IX(l+1)] );
+							if ( match > 0 ) {
+								EXCH(l, l+1);
+							} else if ( match == 0 ) {
+								i = l;
+								break;
+							}
+							i = l+1;
+							j = ir;
+							a = cv[IX(i)];
+							for(;;) {
+								do {
+									i++;
+									COMP( cv[IX(i)], a );
+								} while( match < 0 );
+								while( match > 0 ) {
+									j--;
+									COMP( cv[IX(j)], a );
+								}
+								if (j < i) {
+									match = 1;
+									break;
+								}
+								if ( match == 0 ) {
+									i = l+1;
+									break;
+								}
+								EXCH(i,j);
+							}
+							if ( match == 0 )
+								break;
+							EXCH(l+1,j);
+							jstack += 2;
+							if (ir-i+1 >= j) {
+								istack[jstack] = ir;
+								istack[jstack-1] = i;
+								ir = j;
+							} else {
+								istack[jstack] = j;
+								istack[jstack-1] = l;
+								l = i;
+						}
+					}
+				}
+done:
+				if ( i >= 0 )
+					j = ix[i];
+
+				slap_sl_free( ix, ctx );
+
+				if ( rc != LDAP_SUCCESS ) {
+					return rc;
+				} else if ( match == 0 ) {
+					/* value exists already */
+					assert( i >= 0 );
+					assert( i < nvals );
+					snprintf( textbuf, textlen,
+						"%s: value #%d provided more than once",
+						ml->sml_desc->ad_cname.bv_val, j );
+					*text = textbuf;
+					return LDAP_TYPE_OR_VALUE_EXISTS;
+				}
+#endif	/* SLAP_MODS_CHECK_QUICKSORT */
+			}
 		}
 	}
 
@@ -805,42 +881,54 @@ void slap_mods_opattrs(
 	Modifications *mod, **modtail, *modlast;
 	int gotcsn = 0, gotmname = 0, gotmtime = 0;
 
-	if ( SLAP_LASTMOD( op->o_bd ) ) {
+	if ( SLAP_LASTMOD( op->o_bd ) && !op->orm_no_opattrs ) {
 		char *ptr;
 		timestamp.bv_val = timebuf;
 		for ( modtail = modsp; *modtail; modtail = &(*modtail)->sml_next ) {
 			if ( (*modtail)->sml_op != LDAP_MOD_ADD &&
-				(*modtail)->sml_op != LDAP_MOD_REPLACE ) continue;
-			if ( (*modtail)->sml_desc == slap_schema.si_ad_entryCSN ) {
+				(*modtail)->sml_op != LDAP_MOD_REPLACE )
+			{
+				continue;
+			}
+
+			if ( (*modtail)->sml_desc == slap_schema.si_ad_entryCSN )
+			{
 				csn = (*modtail)->sml_values[0];
 				gotcsn = 1;
-			} else
-			if ( (*modtail)->sml_desc == slap_schema.si_ad_modifiersName ) {
+
+			} else if ( (*modtail)->sml_desc == slap_schema.si_ad_modifiersName )
+			{
 				gotmname = 1;
-			} else
-			if ( (*modtail)->sml_desc == slap_schema.si_ad_modifyTimestamp ) {
+
+			} else if ( (*modtail)->sml_desc == slap_schema.si_ad_modifyTimestamp )
+			{
 				gotmtime = 1;
 			}
 		}
+
 		if ( BER_BVISEMPTY( &op->o_csn )) {
 			if ( !gotcsn ) {
 				csn.bv_val = csnbuf;
 				csn.bv_len = sizeof( csnbuf );
 				slap_get_csn( op, &csn, manage_ctxcsn );
+
 			} else {
-				if ( manage_ctxcsn )
+				if ( manage_ctxcsn ) {
 					slap_queue_csn( op, &csn );
+				}
 			}
+
 		} else {
 			csn = op->o_csn;
 		}
+
 		ptr = ber_bvchr( &csn, '#' );
 		if ( ptr ) {
-			timestamp.bv_len = ptr - csn.bv_val;
-			if ( timestamp.bv_len >= sizeof( timebuf ))	/* ?!? */
-				timestamp.bv_len = sizeof( timebuf ) - 1;
+			timestamp.bv_len = STRLENOF("YYYYMMDDHHMMSSZ");
 			AC_MEMCPY( timebuf, csn.bv_val, timestamp.bv_len );
+			timebuf[timestamp.bv_len-1] = 'Z';
 			timebuf[timestamp.bv_len] = '\0';
+
 		} else {
 			time_t now = slap_get_time();
 
@@ -852,6 +940,7 @@ void slap_mods_opattrs(
 		if ( BER_BVISEMPTY( &op->o_dn ) ) {
 			BER_BVSTR( &name, SLAPD_ANONYMOUS );
 			nname = name;
+
 		} else {
 			name = op->o_dn;
 			nname = op->o_ndn;
@@ -912,3 +1001,117 @@ void slap_mods_opattrs(
 	}
 }
 
+int
+slap_parse_modlist(
+	Operation *op,
+	SlapReply *rs,
+	BerElement *ber,
+	req_modify_s *ms )
+{
+	ber_tag_t	tag;
+	ber_len_t	len;
+	char		*last;
+	Modifications	**modtail = &ms->rs_modlist;
+
+	ms->rs_modlist = NULL;
+	ms->rs_increment = 0;
+
+	rs->sr_err = LDAP_SUCCESS;
+
+	/* collect modifications & save for later */
+	for ( tag = ber_first_element( ber, &len, &last );
+		tag != LBER_DEFAULT;
+		tag = ber_next_element( ber, &len, last ) )
+	{
+		ber_int_t mop;
+		Modifications tmp, *mod;
+
+		tmp.sml_nvalues = NULL;
+
+		if ( ber_scanf( ber, "{e{m[W]}}", &mop,
+		    &tmp.sml_type, &tmp.sml_values ) == LBER_ERROR )
+		{
+			rs->sr_text = "decoding modlist error";
+			rs->sr_err = LDAP_PROTOCOL_ERROR;
+			goto done;
+		}
+
+		mod = (Modifications *) ch_malloc( sizeof(Modifications) );
+		mod->sml_op = mop;
+		mod->sml_flags = 0;
+		mod->sml_type = tmp.sml_type;
+		mod->sml_values = tmp.sml_values;
+		mod->sml_nvalues = NULL;
+		mod->sml_desc = NULL;
+		mod->sml_next = NULL;
+		*modtail = mod;
+
+		switch( mop ) {
+		case LDAP_MOD_ADD:
+			if ( mod->sml_values == NULL ) {
+				Debug( LDAP_DEBUG_ANY, "slap_parse_modlist: "
+					"modify/add operation (%ld) requires values\n",
+					(long) mop, 0, 0 );
+
+				rs->sr_text = "modify/add operation requires values";
+				rs->sr_err = LDAP_PROTOCOL_ERROR;
+				goto done;
+			}
+
+			/* fall through */
+
+		case LDAP_MOD_DELETE:
+		case LDAP_MOD_REPLACE:
+			break;
+
+		case LDAP_MOD_INCREMENT:
+			if( op->o_protocol >= LDAP_VERSION3 ) {
+				ms->rs_increment++;
+				if ( mod->sml_values == NULL ) {
+					Debug( LDAP_DEBUG_ANY, "slap_parse_modlist: "
+						"modify/increment operation (%ld) requires value\n",
+						(long) mop, 0, 0 );
+
+					rs->sr_text = "modify/increment operation requires value";
+					rs->sr_err = LDAP_PROTOCOL_ERROR;
+					goto done;
+				}
+
+				if ( !BER_BVISNULL( &mod->sml_values[ 1 ] ) ) {
+					Debug( LDAP_DEBUG_ANY,  "slap_parse_modlist: modify/increment "
+						"operation (%ld) requires single value\n",
+						(long) mop, 0, 0 );
+
+					rs->sr_text = "modify/increment operation requires single value";
+					rs->sr_err = LDAP_PROTOCOL_ERROR;
+					goto done;
+				}
+
+				break;
+			}
+			/* fall thru */
+
+		default:
+			Debug( LDAP_DEBUG_ANY, "slap_parse_modlist: "
+				"unrecognized modify operation (%ld)\n",
+				(long) mop, 0, 0 );
+
+			rs->sr_text = "unrecognized modify operation";
+			rs->sr_err = LDAP_PROTOCOL_ERROR;
+			goto done;
+		}
+
+		modtail = &mod->sml_next;
+	}
+	*modtail = NULL;
+
+done:
+	if ( rs->sr_err != LDAP_SUCCESS ) {
+		slap_mods_free( ms->rs_modlist, 1 );
+		ms->rs_modlist = NULL;
+		ms->rs_increment = 0;
+	}
+
+	return rs->sr_err;
+}
+
