@@ -1,7 +1,8 @@
 /* dynlist.c - dynamic list overlay */
+/* $OpenLDAP: pkg/ldap/servers/slapd/overlays/dynlist.c,v 1.20.2.4 2007/02/13 20:22:29 kurt Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 2003-2006 The OpenLDAP Foundation.
+ * Copyright 2003-2007 The OpenLDAP Foundation.
  * Portions Copyright 2004-2005 Pierangelo Masarati.
  * All rights reserved.
  *
@@ -22,11 +23,15 @@
 
 #ifdef SLAPD_OVER_DYNLIST
 
-#if LDAP_VENDOR_VERSION_MINOR != X && LDAP_VENDOR_VERSION_MINOR < 3
-#define OL_2_2_COMPAT
-#elif SLAPD_OVER_DYNGROUP != SLAPD_MOD_STATIC
+#if LDAP_VENDOR_VERSION_MINOR == X || LDAP_VENDOR_VERSION_MINOR > 3
+#if SLAPD_OVER_DYNGROUP != SLAPD_MOD_STATIC
 #define TAKEOVER_DYNGROUP
 #endif
+#else
+#if LDAP_VENDOR_VERSION_MINOR < 3
+#define OL_2_2_COMPAT
+#endif
+#endif
 
 #include <stdio.h>
 
@@ -193,7 +198,8 @@ dynlist_sc_update( Operation *op, SlapReply *rs )
 
 	for ( a = rs->sr_entry->e_attrs; a != NULL; a = a->a_next ) {
 		BerVarray	vals, nvals = NULL;
-		int		i, j;
+		int		i, j,
+				is_oc = a->a_desc == slap_schema.si_ad_objectClass;
 
 		/* if attribute is not requested, skip it */
 		if ( rs->sr_attrs == NULL ) {
@@ -242,6 +248,14 @@ dynlist_sc_update( Operation *op, SlapReply *rs )
 		}
 
 		for ( i = 0, j = 0; !BER_BVISNULL( &a->a_vals[i] ); i++ ) {
+			if ( is_oc ) {
+				ObjectClass	*soc = oc_bvfind( &a->a_vals[i] );
+
+				if ( soc->soc_kind == LDAP_SCHEMA_STRUCTURAL ) {
+					continue;
+				}
+			}
+
 			if ( access_allowed( op, rs->sr_entry, a->a_desc,
 						&a->a_nvals[i], ACL_READ, &acl_state ) )
 			{
@@ -284,6 +298,8 @@ dynlist_sc_update( Operation *op, SlapReply *rs )
 done:;
 	if ( rs->sr_flags & REP_ENTRY_MUSTBEFREED ) {
 		entry_free( rs->sr_entry );
+		rs->sr_entry = NULL;
+		rs->sr_flags ^= REP_ENTRY_MUSTBEFREED;
 	}
 
 	return 0;
@@ -350,7 +366,7 @@ dynlist_send_entry( Operation *op, SlapReply *rs, dynlist_info_t *dli )
 			continue;
 		}
 
-		if ( lud->lud_host ) {
+		if ( lud->lud_host != NULL ) {
 			/* FIXME: host not allowed; reject as illegal? */
 			Debug( LDAP_DEBUG_ANY, "dynlist_send_entry(\"%s\"): "
 				"illegal URI \"%s\"\n",
@@ -365,6 +381,7 @@ dynlist_send_entry( Operation *op, SlapReply *rs, dynlist_info_t *dli )
 			 * this can be useful in case of a database serving
 			 * the empty suffix */
 			BER_BVSTR( &dn, "" );
+
 		} else {
 			ber_str2bv( lud->lud_dn, 0, 0, &dn );
 		}
@@ -435,6 +452,7 @@ dynlist_send_entry( Operation *op, SlapReply *rs, dynlist_info_t *dli )
 		if ( lud->lud_filter == NULL ) {
 			ber_dupbv_x( &o.ors_filterstr,
 					&dli->dli_default_filter, op->o_tmpmemctx );
+
 		} else {
 			struct berval	flt;
 			ber_str2bv( lud->lud_filter, 0, 0, &flt );
@@ -471,10 +489,8 @@ cleanup:;
 		if ( !BER_BVISNULL( &o.o_req_ndn ) ) {
 			op->o_tmpfree( o.o_req_ndn.bv_val, op->o_tmpmemctx );
 		}
-		if ( o.ors_filterstr.bv_val != lud->lud_filter ) {
-			op->o_tmpfree( o.ors_filterstr.bv_val, op->o_tmpmemctx );
-			lud->lud_filter = NULL;
-		}
+		assert( o.ors_filterstr.bv_val != lud->lud_filter );
+		op->o_tmpfree( o.ors_filterstr.bv_val, op->o_tmpmemctx );
 		ldap_free_urldesc( lud );
 	}
 
@@ -607,6 +623,8 @@ dynlist_compare( Operation *op, SlapReply *rs )
 		o.ors_attrs = an;
 		o.ors_attrsonly = 0;
 
+		o.o_acl_priv = ACL_COMPARE;
+
 		rc = o.o_bd->be_search( &o, &r );
 		filter_free_x( &o, o.ors_filter );
 
@@ -694,7 +712,7 @@ dynlist_build_def_filter( dynlist_info_t *dli )
 
 	dli->dli_default_filter.bv_len = STRLENOF( "(!(objectClass=" "))" )
 		+ dli->dli_oc->soc_cname.bv_len;
-	dli->dli_default_filter.bv_val = SLAP_MALLOC( dli->dli_default_filter.bv_len + 1 );
+	dli->dli_default_filter.bv_val = ch_malloc( dli->dli_default_filter.bv_len + 1 );
 	if ( dli->dli_default_filter.bv_val == NULL ) {
 		Debug( LDAP_DEBUG_ANY, "dynlist_db_open: malloc failed.\n",
 			0, 0, 0 );
@@ -1343,9 +1361,11 @@ dynlist_db_open(
 			dli->dli_ad = ad;			
 		}
 
-		rc = dynlist_build_def_filter( dli );
-		if ( rc != 0 ) {
-			return rc;
+		if ( BER_BVISNULL( &dli->dli_default_filter ) ) {
+			rc = dynlist_build_def_filter( dli );
+			if ( rc != 0 ) {
+				return rc;
+			}
 		}
 	}
 
