@@ -1,7 +1,7 @@
-/* $OpenLDAP: pkg/ldap/servers/slapd/overlays/pcache.c,v 1.88.2.1 2006/08/17 23:50:32 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/overlays/pcache.c,v 1.88.2.3 2007/02/13 20:22:30 kurt Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 2003-2006 The OpenLDAP Foundation.
+ * Copyright 2003-2007 The OpenLDAP Foundation.
  * Portions Copyright 2003 IBM Corporation.
  * Portions Copyright 2003 Symas Corporation.
  * All rights reserved.
@@ -216,7 +216,8 @@ merge_entry(
 			rc = 0;
 		}
 	} else {
-		be_entry_release_w( op, e );
+		if ( op->ora_e == e )
+			be_entry_release_w( op, e );
 		rc = 1;
 	}
 
@@ -1925,7 +1926,7 @@ pc_cf_gen( ConfigArgs *c )
 			return( 1 );
 		}
 
-		if ( !backend_db_init( c->argv[1], &cm->db )) {
+		if ( !backend_db_init( c->argv[1], &cm->db, -1 )) {
 			snprintf( c->msg, sizeof( c->msg ), "unknown backend type (arg #1)" );
 			Debug( LDAP_DEBUG_CONFIG, "%s: %s.\n", c->log, c->msg, 0 );
 			return( 1 );
@@ -2128,8 +2129,7 @@ pcache_db_config(
 
 static int
 pcache_db_init(
-	BackendDB *be
-)
+	BackendDB *be )
 {
 	slap_overinst *on = (slap_overinst *)be->bd_info;
 	cache_manager *cm;
@@ -2171,8 +2171,7 @@ pcache_db_init(
 
 static int
 pcache_db_open(
-	BackendDB *be
-)
+	BackendDB *be )
 {
 	slap_overinst	*on = (slap_overinst *)be->bd_info;
 	cache_manager	*cm = on->on_bi.bi_private;
@@ -2213,6 +2212,13 @@ pcache_db_open(
 	cm->db.be_acl = be->be_acl;
 	cm->db.be_dfltaccess = be->be_dfltaccess;
 
+	if ( SLAP_DBMONITORING( be ) ) {
+		SLAP_DBFLAGS( &cm->db ) |= SLAP_DBFLAG_MONITORING;
+
+	} else {
+		SLAP_DBFLAGS( &cm->db ) &= ~SLAP_DBFLAG_MONITORING;
+	}
+
 	rc = backend_startup_one( &cm->db );
 
 	/* There is no runqueue in TOOL mode */
@@ -2301,16 +2307,8 @@ pcache_db_destroy(
 	cache_manager *cm = on->on_bi.bi_private;
 	query_manager *qm = cm->qm;
 
-	/* cleanup stuff inherited from the original database... */
-	cm->db.be_suffix = NULL;
-	cm->db.be_nsuffix = NULL;
-	BER_BVZERO( &cm->db.be_rootdn );
-	BER_BVZERO( &cm->db.be_rootndn );
-	BER_BVZERO( &cm->db.be_rootpw );
-	/* FIXME: there might be more... */
-
 	if ( cm->db.be_private != NULL ) {
-		backend_destroy_one( &cm->db, 0 );
+		backend_stopdown_one( &cm->db );
 	}
 
 	ldap_pvt_thread_mutex_destroy( &qm->lru_mutex );
@@ -2330,31 +2328,16 @@ static char *obsolete_names[] = {
 
 int pcache_initialize()
 {
-	LDAPAttributeType *at;
 	int code;
-	const char *err;
 	struct berval debugbv = BER_BVC("pcache");
 
 	if (( code = slap_loglevel_get( &debugbv, &pcache_debug )))
 		return code;
 
-	at = ldap_str2attributetype( queryid_schema, &code, &err,
-		LDAP_SCHEMA_ALLOW_ALL );
-	if ( !at ) {
-		Debug( LDAP_DEBUG_ANY,
-			"pcache_initialize: ldap_str2attributetype failed %s %s\n",
-			ldap_scherr2str(code), err, 0 );
-		return code;
-	}
-	code = at_add( at, 0, NULL, &err );
-	if ( !code ) {
-		slap_str2ad( at->at_names[0], &ad_queryid, &err );
-	}
-	ldap_memfree( at );
+	code = register_at( queryid_schema, &ad_queryid, 0 );
 	if ( code ) {
 		Debug( LDAP_DEBUG_ANY,
-			"pcache_initialize: at_add failed %s %s\n",
-			scherr2str(code), err, 0 );
+			"pcache_initialize: register_at failed\n", 0, 0, 0 );
 		return code;
 	}
 
