@@ -1,8 +1,8 @@
 /* root_dse.c - Provides the Root DSA-Specific Entry */
-/* $OpenLDAP: pkg/ldap/servers/slapd/root_dse.c,v 1.113.2.4 2006/08/18 04:30:12 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/root_dse.c,v 1.113.2.6 2007/02/13 20:22:20 kurt Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 1999-2006 The OpenLDAP Foundation.
+ * Copyright 1999-2007 The OpenLDAP Foundation.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -213,10 +213,10 @@ root_dse_info(
 	AttributeDescription *ad_ref
 		= slap_schema.si_ad_ref;
 
-	e = (Entry *) SLAP_CALLOC( 1, sizeof(Entry) );
+	e = entry_alloc();
 	if( e == NULL ) {
 		Debug( LDAP_DEBUG_ANY,
-			"root_dse_info: SLAP_CALLOC failed", 0, 0, 0 );
+			"root_dse_info: entry_alloc failed", 0, 0, 0 );
 		return LDAP_OTHER;
 	}
 
@@ -322,7 +322,7 @@ fail:
 		 * clients.
 		 */
 	for ( i=LDAP_VERSION3; i<=LDAP_VERSION_MAX; i++ ) {
-		char buf[BUFSIZ];
+		char buf[sizeof("255")];
 		snprintf(buf, sizeof buf, "%d", i);
 		val.bv_val = buf;
 		val.bv_len = strlen( val.bv_val );
@@ -375,13 +375,31 @@ fail:
 	return LDAP_SUCCESS;
 }
 
+int
+root_dse_init( void )
+{
+	return 0;
+}
+
+int
+root_dse_destroy( void )
+{
+	if ( usr_attr ) {
+		entry_free( usr_attr );
+		usr_attr = NULL;
+	}
+
+	return 0;
+}
+
 /*
  * Read the entries specified in fname and merge the attributes
  * to the user defined rootDSE. Note thaat if we find any errors
  * what so ever, we will discard the entire entries, print an
  * error message and return.
  */
-int read_root_dse_file( const char *fname )
+int
+root_dse_read_file( const char *fname )
 {
 	struct LDIFFP	*fp;
 	int rc = 0, lineno = 0, lmax = 0;
@@ -389,16 +407,16 @@ int read_root_dse_file( const char *fname )
 
 	if ( (fp = ldif_open( fname, "r" )) == NULL ) {
 		Debug( LDAP_DEBUG_ANY,
-			"could not open rootdse attr file \"%s\" - absolute path?\n",
+			"root_dse_read_file: could not open rootdse attr file \"%s\" - absolute path?\n",
 			fname, 0, 0 );
 		perror( fname );
 		return EXIT_FAILURE;
 	}
 
-	usr_attr = (Entry *) SLAP_CALLOC( 1, sizeof(Entry) );
+	usr_attr = entry_alloc();
 	if( usr_attr == NULL ) {
 		Debug( LDAP_DEBUG_ANY,
-			"read_root_dse_file: SLAP_CALLOC failed", 0, 0, 0 );
+			"root_dse_read_file: entry_alloc failed", 0, 0, 0 );
 		ldif_close( fp );
 		return LDAP_OTHER;
 	}
@@ -409,17 +427,19 @@ int read_root_dse_file( const char *fname )
 		Attribute *a;
 
 		if( e == NULL ) {
-			fprintf( stderr, "root_dse: could not parse entry (line=%d)\n",
-				lineno );
+			Debug( LDAP_DEBUG_ANY, "root_dse_read_file: "
+				"could not parse entry (file=\"%s\" line=%d)\n",
+				fname, lineno, 0 );
 			rc = EXIT_FAILURE;
 			break;
 		}
 
 		/* make sure the DN is the empty DN */
 		if( e->e_nname.bv_len ) {
-			fprintf( stderr,
-				"root_dse: invalid rootDSE - dn=\"%s\" (line=%d)\n",
-				e->e_dn, lineno );
+			Debug( LDAP_DEBUG_ANY,
+				"root_dse_read_file: invalid rootDSE "
+				"- dn=\"%s\" (file=\"%s\" line=%d)\n",
+				e->e_dn, fname, lineno );
 			entry_free( e );
 			rc = EXIT_FAILURE;
 			break;
@@ -453,45 +473,29 @@ int read_root_dse_file( const char *fname )
 
 	ldif_close( fp );
 
-	Debug(LDAP_DEBUG_CONFIG, "rootDSE file %s read.\n", fname, 0, 0);
+	Debug(LDAP_DEBUG_CONFIG, "rootDSE file=\"%s\" read.\n", fname, 0, 0);
 	return rc;
 }
 
 int
 slap_discover_feature(
-	const char	*uri,
-	int		version,
+	slap_bindconf	*sb,
 	const char	*attr,
 	const char	*val )
 {
-	LDAP		*ld;
+	LDAP		*ld = NULL;
 	LDAPMessage	*res = NULL, *entry;
 	int		rc, i;
-	struct berval	cred = BER_BVC( "" ),
-			bv_val,
+	struct berval	bv_val,
 			**values = NULL;
 	char		*attrs[ 2 ] = { NULL, NULL };
 
-	ber_str2bv( val, 0, 0, &bv_val );
+	rc = slap_client_connect( &ld, sb );
+	if ( rc != LDAP_SUCCESS ) {
+		goto done;
+	}
+
 	attrs[ 0 ] = (char *) attr;
-
-	rc = ldap_initialize( &ld, uri );
-	if ( rc != LDAP_SUCCESS ) {
-		return rc;
-	}
-
-	rc = ldap_set_option( ld, LDAP_OPT_PROTOCOL_VERSION,
-		(const void *)&version );
-	if ( rc != LDAP_SUCCESS ) {
-		goto done;
-	}
-
-	rc = ldap_sasl_bind_s( ld, "", LDAP_SASL_SIMPLE,
-			&cred, NULL, NULL, NULL );
-	if ( rc != LDAP_SUCCESS ) {
-		goto done;
-	}
-
 	rc = ldap_search_ext_s( ld, "", LDAP_SCOPE_BASE, "(objectClass=*)",
 			attrs, 0, NULL, NULL, NULL, 0, &res );
 	if ( rc != LDAP_SUCCESS ) {
@@ -509,6 +513,7 @@ slap_discover_feature(
 		goto done;
 	}
 
+	ber_str2bv( val, 0, 0, &bv_val );
 	for ( i = 0; values[ i ] != NULL; i++ ) {
 		if ( bvmatch( &bv_val, values[ i ] ) ) {
 			rc = LDAP_COMPARE_TRUE;
