@@ -1,8 +1,8 @@
 /* sl_malloc.c - malloc routines using a per-thread slab */
-/* $OpenLDAP: pkg/ldap/servers/slapd/sl_malloc.c,v 1.39 2006/04/06 17:13:41 ando Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/sl_malloc.c,v 1.39.2.2 2007/02/15 00:44:18 kurt Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 2003-2006 The OpenLDAP Foundation.
+ * Copyright 2003-2007 The OpenLDAP Foundation.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -94,7 +94,8 @@ void *
 slap_sl_mem_create(
 	ber_len_t size,
 	int stack,
-	void *ctx
+	void *ctx,
+	int new
 )
 {
 	struct slab_heap *sh;
@@ -113,6 +114,9 @@ slap_sl_mem_create(
 	sh = sh_tmp;
 #endif
 
+	if ( !new )
+		return sh;
+
 	/* round up to doubleword boundary */
 	size += pad;
 	size &= ~pad;
