@@ -1,7 +1,7 @@
-/* $OpenLDAP: pkg/ldap/servers/slapd/slapadd.c,v 1.36.2.1 2006/08/17 23:50:28 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/slapadd.c,v 1.36.2.2 2007/02/13 20:22:22 kurt Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 1998-2006 The OpenLDAP Foundation.
+ * Copyright 1998-2007 The OpenLDAP Foundation.
  * Portions Copyright 1998-2003 Kurt D. Zeilenga.
  * Portions Copyright 2003 IBM Corporation.
  * All rights reserved.
@@ -71,6 +71,7 @@ slapadd( int argc, char **argv )
 
 	memset( &opbuf, 0, sizeof(opbuf) );
 	op = (Operation *) &opbuf;
+	op->o_hdr = (Opheader *)(op+1);
 
 	if( !be->be_entry_open ||
 		!be->be_entry_close ||
@@ -163,8 +164,6 @@ slapadd( int argc, char **argv )
 		}
 
 		{
-			Attribute *sc = attr_find( e->e_attrs,
-				slap_schema.si_ad_structuralObjectClass );
 			Attribute *oc = attr_find( e->e_attrs,
 				slap_schema.si_ad_objectClass );
 
@@ -178,30 +177,11 @@ slapadd( int argc, char **argv )
 				break;
 			}
 
-			if( sc == NULL ) {
-				struct berval val;
-
-				rc = structural_class( oc->a_vals, &val,
-					NULL, &text, textbuf, textlen );
-
-				if( rc != LDAP_SUCCESS ) {
-					fprintf( stderr, "%s: dn=\"%s\" (line=%d): (%d) %s\n",
-						progname, e->e_dn, lineno, rc, text );
-					rc = EXIT_FAILURE;
-					entry_free( e );
-					if( continuemode ) continue;
-					break;
-				}
-
-				attr_merge_one( e, slap_schema.si_ad_structuralObjectClass,
-					&val, NULL );
-			}
-
 			/* check schema */
 			op->o_bd = be;
 
 			if ( (slapMode & SLAP_TOOL_NO_SCHEMA_CHECK) == 0) {
-				rc = entry_schema_check( op, e, NULL, manage,
+				rc = entry_schema_check( op, e, NULL, manage, 1,
 					&text, textbuf, textlen );
 
 				if( rc != LDAP_SUCCESS ) {
