@@ -1,7 +1,7 @@
-/* $OpenLDAP: pkg/ldap/servers/slapd/slapi/slapi_pblock.c,v 1.63.2.2 2006/08/17 23:50:32 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/slapi/slapi_pblock.c,v 1.63.2.4 2007/02/13 20:22:32 kurt Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 2002-2006 The OpenLDAP Foundation.
+ * Copyright 2002-2007 The OpenLDAP Foundation.
  * Portions Copyright 1997,2002-2003 IBM Corporation.
  * All rights reserved.
  *
@@ -64,7 +64,7 @@ pblock_get_param_class( int param )
 	case SLAPI_DB2LDIF_PRINTKEY:
 	case SLAPI_LDIF2DB_REMOVEDUPVALS:
 	case SLAPI_MANAGEDSAIT:
-	case SLAPI_X_MANAGEDIT:
+	case SLAPI_X_RELAX:
 	case SLAPI_X_OPERATION_NO_SCHEMA_CHECK:
 	case SLAPI_IS_REPLICATED_OPERATION:
 	case SLAPI_X_CONN_IS_UDP:
@@ -497,7 +497,8 @@ pblock_get( Slapi_PBlock *pb, int param, void **value )
 
 			rc = mods_structural_class( pb->pb_op->ora_modlist,
 				&tmpval, &pb->pb_rs->sr_text,
-				pb->pb_textbuf, sizeof( pb->pb_textbuf ));
+				pb->pb_textbuf, sizeof( pb->pb_textbuf ),
+				pb->pb_op->o_tmpmemctx );
 			*((char **)value) = tmpval.bv_val;
 		} else {
 			rc = PBLOCK_ERROR;
@@ -519,9 +520,9 @@ pblock_get( Slapi_PBlock *pb, int param, void **value )
 		PBLOCK_ASSERT_OP( pb, 0 );
 		*((int *)value) = get_manageDSAit( pb->pb_op );
 		break;
-	case SLAPI_X_MANAGEDIT:
+	case SLAPI_X_RELAX:
 		PBLOCK_ASSERT_OP( pb, 0 );
-		*((int *)value) = get_manageDIT( pb->pb_op );
+		*((int *)value) = get_relax( pb->pb_op );
 		break;
 	case SLAPI_BACKEND:
 		PBLOCK_ASSERT_OP( pb, 0 );
@@ -924,9 +925,9 @@ pblock_set( Slapi_PBlock *pb, int param, void *value )
 		PBLOCK_ASSERT_OP( pb, 0 );
 		pb->pb_op->o_managedsait = *((int *)value);
 		break;
-	case SLAPI_X_MANAGEDIT:
+	case SLAPI_X_RELAX:
 		PBLOCK_ASSERT_OP( pb, 0 );
-		pb->pb_op->o_managedit = *((int *)value);
+		pb->pb_op->o_relax = *((int *)value);
 		break;
 	case SLAPI_BACKEND:
 		PBLOCK_ASSERT_OP( pb, 0 );
@@ -1004,7 +1005,7 @@ pblock_set( Slapi_PBlock *pb, int param, void *value )
 			break;
 		}
 
-		newmods = slapi_int_ldapmods2modifications( (LDAPMod **)value );
+		newmods = slapi_int_ldapmods2modifications( pb->pb_op, (LDAPMod **)value );
 		if ( newmods != NULL ) {
 			slap_mods_free( *mlp, 1 );
 			*mlp = newmods;
