@@ -1,7 +1,7 @@
-/* $OpenLDAP: pkg/ldap/servers/slurpd/ri.c,v 1.35 2006/01/03 22:12:26 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slurpd/ri.c,v 1.35.2.2 2007/02/13 20:22:32 kurt Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 1998-2006 The OpenLDAP Foundation.
+ * Copyright 1998-2007 The OpenLDAP Foundation.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -121,7 +121,7 @@ Ri_process(
 		case DO_LDAP_ERR_FATAL: {
 		    /* Non-retryable error.  Write rejection log. */
 			int ld_errno = 0;
-			ldap_get_option(ri->ri_ldp, LDAP_OPT_ERROR_NUMBER, &ld_errno);
+			ldap_get_option(ri->ri_ldp, LDAP_OPT_RESULT_CODE, &ld_errno);
 		    write_reject( ri, re, ld_errno, errmsg );
 		    /* Update status ... */
 		    (void) sglob->st->st_update( sglob->st, ri->ri_stel, re );
@@ -209,7 +209,6 @@ Ri_init(
     (*ri)->ri_bind_dn = NULL;
     (*ri)->ri_password = NULL;
     (*ri)->ri_authcId = NULL;
-    (*ri)->ri_srvtab = NULL;
     (*ri)->ri_curr = NULL;
 
     return 0;
