@@ -1,7 +1,7 @@
-/* $OpenLDAP: pkg/ldap/tests/progs/slapd-common.c,v 1.4.2.2 2006/08/17 23:50:33 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/tests/progs/slapd-common.c,v 1.4.2.4 2007/02/13 20:22:35 kurt Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 1999-2006 The OpenLDAP Foundation.
+ * Copyright 1999-2007 The OpenLDAP Foundation.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -31,6 +31,10 @@
 #include "ldap_pvt.h"
 #include "slapd-common.h"
 
+/* global vars */
+pid_t pid;
+
+/* static vars */
 static char progname[ BUFSIZ ];
 tester_t progtype;
 
@@ -208,7 +212,9 @@ tester_ignore_err( int err )
 void
 tester_init( const char *pname, tester_t ptype )
 {
-	snprintf( progname, sizeof( progname ), "%s PID=%d", pname, getpid() );
+	pid = getpid();
+	srand( pid );
+	snprintf( progname, sizeof( progname ), "%s PID=%d", pname, pid );
 	progtype = ptype;
 }
 
@@ -235,7 +241,7 @@ tester_ldap_error( LDAP *ld, const char *fname, const char *msg )
 
 	ldap_get_option( ld, LDAP_OPT_RESULT_CODE, (void *)&err );
 	if ( err != LDAP_SUCCESS ) {
-		ldap_get_option( ld, LDAP_OPT_ERROR_STRING, (void *)&text );
+		ldap_get_option( ld, LDAP_OPT_DIAGNOSTIC_MESSAGE, (void *)&text );
 	}
 
 	fprintf( stderr, "%s: %s: %s (%d) %s %s\n",
