@@ -1,7 +1,7 @@
-/* $OpenLDAP: pkg/ldap/tests/progs/slapd-common.h,v 1.2.2.2 2006/08/17 23:50:33 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/tests/progs/slapd-common.h,v 1.2.2.3 2007/02/13 20:22:35 kurt Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 1999-2006 The OpenLDAP Foundation.
+ * Copyright 1999-2007 The OpenLDAP Foundation.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -27,7 +27,8 @@ typedef enum {
 	TESTER_MODIFY,
 	TESTER_MODRDN,
 	TESTER_READ,
-	TESTER_SEARCH
+	TESTER_SEARCH,
+	TESTER_LAST
 } tester_t;
 
 extern void tester_init( const char *pname, tester_t ptype );
@@ -38,4 +39,6 @@ extern void tester_ldap_error( LDAP *ld, const char *fname, const char *msg );
 extern int tester_ignore_str2errlist( const char *err );
 extern unsigned tester_ignore_err( int err );
 
+extern pid_t		pid;
+
 #endif /* SLAPD_COMMON_H */
