@@ -1,7 +1,7 @@
-/* $OpenLDAP: pkg/ldap/tests/progs/slapd-modify.c,v 1.19.2.2 2006/08/17 23:50:33 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/tests/progs/slapd-modify.c,v 1.19.2.3 2007/02/13 20:22:35 kurt Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 1999-2006 The OpenLDAP Foundation.
+ * Copyright 1999-2007 The OpenLDAP Foundation.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -81,7 +81,8 @@ main( int argc, char **argv )
 
 	tester_init( "slapd-modify", TESTER_MODIFY );
 
-	while ( (i = getopt( argc, argv, "CFH:h:i:p:D:w:e:a:l:L:r:t:" )) != EOF ) {
+	while ( ( i = getopt( argc, argv, "a:CD:e:FH:h:i:L:l:p:r:t:w:" ) ) != EOF )
+	{
 		switch ( i ) {
 		case 'C':
 			chaserefs++;
@@ -200,7 +201,6 @@ do_modify( char *uri, char *manager,
 {
 	LDAP	*ld = NULL;
 	int  	i = 0, do_retry = maxretries;
-	pid_t	pid;
 	int     rc = LDAP_SUCCESS;
 
 	struct ldapmod mod;
@@ -208,8 +208,6 @@ do_modify( char *uri, char *manager,
 	char *values[2];
 	int version = LDAP_VERSION3;
 
-	pid = getpid();
-	
 	values[0] = value;
 	values[1] = NULL;
 	mod.mod_op = LDAP_MOD_ADD;
